
import unittest
from .currency import usd_to_aud
from .currency_usd_to_cad import usd_to_cad


class CurrencyTestCase(unittest.TestCase):

    def test_usd_to_aud(self):
        item = {'body_html': '$100'}
        res = usd_to_aud(item)
        aud = float(res['body_html'][3:])
        self.assertGreater(aud, 100)  # if this fails, check first the currency ;)

    def test_usd_to_cad(self):
        item = {'body_html': '$100'}
        res = usd_to_cad(item)
        cad = float(res['body_html'][3:])
        self.assertGreater(cad, 100)  # if this fails, check first the currency ;)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
from datetime import datetime

import superdesk
from superdesk.celery_app import celery
from superdesk.utc import utcnow
import superdesk.publish
from superdesk.errors import PublishQueueError
from superdesk.celery_task_utils import is_task_running, mark_task_as_not_running
from superdesk import get_resource_service

logger = logging.getLogger(__name__)

UPDATE_SCHEDULE_DEFAULT = {'seconds': 10}
PUBLISH_QUEUE = 'publish_queue'
STATE_PENDING = 'pending'


class PublishContent(superdesk.Command):
    """Runs deliveries"""

    def run(self, provider_type=None):
        publish.apply_async(expires=10)


@celery.task(soft_time_limit=1800)
def publish():
    """
    Fetches items from publish queue as per the configuration,
    calls the transmit function.
    """

    if is_task_running("Transmit", "Articles", UPDATE_SCHEDULE_DEFAULT):
        return

    try:
        items = get_queue_items()

        if items.count() > 0:
            transmit_items(items)
    finally:
        mark_task_as_not_running("Transmit", "Articles")


def get_queue_items():
    lookup = {'$and': [{'state': STATE_PENDING}, {'destination.delivery_type': {'$ne': 'pull'}}]}
    return get_resource_service(PUBLISH_QUEUE).get(req=None, lookup=lookup)


def transmit_items(queue_items):
    publish_queue_service = get_resource_service(PUBLISH_QUEUE)
    failed_items = {}

    for queue_item in queue_items:
        try:
            if not is_on_time(queue_item):
                continue

            # update the status of the item to in-progress
            queue_update = {'state': 'in-progress', 'transmit_started_at': utcnow()}
            publish_queue_service.patch(queue_item.get('_id'), queue_update)

            destination = queue_item['destination']

            transmitter = superdesk.publish.registered_transmitters[destination.get('delivery_type')]
            transmitter.transmit(queue_item)
            update_content_state(queue_item)
        except Exception as ex:
            logger.exception(ex)
            failed_items[str(queue_item.get('_id'))] = queue_item

    # mark failed items as pending so that Celery tasks will try again
    if len(failed_items) > 0:
        for item_id in failed_items.keys():
            try:
                orig_item = publish_queue_service.find_one(item_id)
                publish_queue_service.system_update(
                    item_id, {'state': STATE_PENDING}, orig_item)
            except:
                logger.error(
                    'Failed to set the publish queue item back to "pending" '
                    'state: {}'.format(item_id))

        logger.error('Failed to publish the following items: {}'.format(failed_items.keys()))


def is_on_time(queue_item):
    """
    Checks if the item is ready to be processed

    :param queue_item: item to be checked
    :return: True if the item is ready
    """

    try:
        if queue_item.get('publish_schedule'):
            publish_schedule = queue_item['publish_schedule']
            if type(publish_schedule) is not datetime:
                raise PublishQueueError.bad_schedule_error(Exception("Schedule is not datetime"), queue_item['_id'])
            return utcnow() >= publish_schedule

        return True
    except PublishQueueError:
        raise
    except Exception as ex:
        raise PublishQueueError.bad_schedule_error(ex, queue_item['destination'])


def update_content_state(queue_item):
    """
    Updates the state of the content item to published, in archive and published collections.
    """

    if queue_item.get('publish_schedule'):
        try:
            item_update = {'state': 'published'}
            get_resource_service('archive').patch(queue_item['item_id'], item_update)
            get_resource_service('published').update_published_items(queue_item['item_id'], 'state', 'published')
        except Exception as ex:
            raise PublishQueueError.content_update_error(ex)

superdesk.command('publish:transmit', PublishContent())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from operator import itemgetter

from eve.render import send_response
from flask.blueprints import Blueprint

import superdesk
from superdesk.utc import utcnow
from superdesk.locators import locators

bp = Blueprint('locators', __name__)


@bp.route('/cities/', methods=['GET', 'OPTIONS'])
@bp.route('/country/<country_code>', methods=['GET', 'OPTIONS'])
@bp.route('/country/<country_code>/state/<state_code>', methods=['GET', 'OPTIONS'])
def get_cities(country_code=None, state_code=None):
    """
    Fetches cities and sends the list as response body.

    :param  country_code: Country Code as defined in the locators.json file.
            For example, AU for Australia, NZ for New Zealand.
    :param state_code: State Code as defined in the locators.json file. For example, VIC for Victoria, NV for Nevada.
    :return:
        Returns HTTP Response with body {'_items': cities, '_meta': {'total': City Count}}.
    """

    cities = find_cities(country_code=country_code, state_code=state_code)

    if cities and len(cities):
        response_data = {'_items': cities, '_meta': {'total': len(cities)}}
        return send_response(None, (response_data, utcnow(), None, 200))
    else:
        return send_response(None, ({}, utcnow(), None, 404))


def find_cities(country_code=None, state_code=None):
    """
    Fetches cities in the locators.json file.

    Returns all the cities in the locators.json file.
    When country_code is passed it gets cities in the country identified by country_code.

    :param  country_code: Country Code as defined in the locators.json file.
            For example, AU for Australia, NZ for New Zealand.
    :param state_code: State Code as defined in the locators.json file. For example, VIC for Victoria, NV for Nevada.
    :return:
        Returns Cities in a State when both country_code and state_code are passed.
        Returns Cities in a Country when only country_code is passed.
        Returns Cities across all countries when nothing is passed.
    """

    if country_code and state_code:
        cities = [{'city_code': city['qcode'], 'city': city['name'], 'alt_name': city.get('alt_name', ''),
                   'tz': city['timezone'], 'dateline': city['dateline_format'], 'state': state['name'],
                   'state_code': state['qcode'], 'country': country['name'], 'country_code': country['qcode']}
                  for country in locators if country['qcode'] == country_code
                  for state in country['states'] if state['qcode'] == state_code for city in state['cities']]
        cities.sort(key=itemgetter('city'))
    elif country_code:
        cities = [{'city_code': city['qcode'], 'city': city['name'], 'alt_name': city.get('alt_name', ''),
                   'tz': city['timezone'], 'dateline': city['dateline_format'], 'state': state['name'],
                   'state_code': state['qcode'], 'country': country['name'], 'country_code': country['qcode']}
                  for country in locators if country['qcode'] == country_code for state in country['states']
                  for city in state['cities']]
        cities.sort(key=itemgetter('state', 'city'))
    else:
        cities = [{'city_code': city['qcode'], 'city': city['name'], 'alt_name': city.get('alt_name', ''),
                   'tz': city['timezone'], 'dateline': city['dateline_format'], 'state': state['name'],
                   'state_code': state['qcode'], 'country': country['name'], 'country_code': country['qcode']}
                  for country in locators for state in country['states'] for city in state['cities']]
        cities.sort(key=itemgetter('country', 'state', 'city'))

    return cities


superdesk.blueprint(bp)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.publish.formatters import Formatter
from apps.publish.formatters.aap_formatter_common import map_priority
import superdesk
from bs4 import BeautifulSoup
from superdesk.errors import FormatterError
from superdesk.metadata.item import ITEM_TYPE, CONTENT_TYPE, EMBARGO


class AAPSMSFormatter(Formatter):
    def format(self, article, subscriber):
        """
        Constructs a dictionary that represents the parameters passed to the SMS InsertAlerts stored procedure
        :return: returns the sequence number of the subscriber and the constructed parameter dictionary
        """
        try:
            pub_seq_num = superdesk.get_resource_service('subscribers').generate_sequence_number(subscriber)

            odbc_item = {'Sequence': pub_seq_num, 'Category': article.get('anpa_category', [{}])[0].get('qcode'),
                         'Headline': article.get('headline', '').replace('\'', '\'\''),
                         'Priority': map_priority(article.get('priority'))}

            body = self.append_body_footer(article)
            if article.get(EMBARGO):
                embargo = '{}{}'.format('Embargo Content. Timestamp: ', article.get(EMBARGO).isoformat())
                body = embargo + body

            if article[ITEM_TYPE] == CONTENT_TYPE.TEXT:
                body = BeautifulSoup(body, "html.parser").text

            odbc_item['StoryText'] = body.replace('\'', '\'\'')  # @article_text
            odbc_item['ident'] = '0'

            return [(pub_seq_num, odbc_item)]
        except Exception as ex:
            raise FormatterError.AAPSMSFormatterError(ex, subscriber)

    def can_format(self, format_type, article):
        # need to check that a story with the same headline has not been published to SMS before
        lookup = {'destination.format': format_type, 'headline': article['headline']}
        published = superdesk.get_resource_service('publish_queue').get(req=None, lookup=lookup)
        if published and published.count():
            return False
        return format_type == 'AAP SMS' and article[ITEM_TYPE] in [CONTENT_TYPE.TEXT, CONTENT_TYPE.PREFORMATTED]



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
from abc import ABCMeta, abstractmethod
from datetime import datetime

from pytz import utc

import superdesk
from superdesk import get_resource_service
from superdesk.errors import SuperdeskApiError, SuperdeskIngestError
from superdesk.io import registered_feed_parsers
from superdesk.utc import utcnow

logger = logging.getLogger(__name__)


class FeedingService(metaclass=ABCMeta):
    """
    Base Class for all Feeding Service classes.

    A Feeding Service class must have the following attributes:
        1. `NAME` - unique name under which to register the class.
        2. `ERRORS` - representing a list of <error_number, error_message> pairs that might be raised by the class
                      instances' methods.
    """

    @abstractmethod
    def _update(self, provider):
        """
        Subclasses must override this method and get items from the provider as per the configuration.

        :param provider: Ingest Provider Details.
                .. seealso:: :class: `superdesk.io.ingest_provider_model.IngestProviderResource`
        :type provider: dict
        :return: a list of articles which can be saved in Ingest Collection.
        """

        raise NotImplementedError()

    def update(self, provider):
        """
        Clients consuming Ingest Services should invoke this to get items from the provider.

        :param provider: Ingest Provider Details.
        :type provider: dict :py:class: `superdesk.io.ingest_provider_model.IngestProviderResource`
        :return: a list of articles which can be saved in Ingest Collection.
        :raises SuperdeskApiError.internalError if Provider is closed
        :raises SuperdeskIngestError if failed to get items from provider
        """

        is_closed = provider.get('is_closed', False)

        if isinstance(is_closed, datetime):
            is_closed = False

        if is_closed:
            raise SuperdeskApiError.internalError('Ingest Provider is closed')
        else:
            try:
                return self._update(provider) or []
            except SuperdeskIngestError as error:
                self.close_provider(provider, error)
                raise error

    def close_provider(self, provider, error):
        """
        Closes the provider and uses error as reason for closing.
        :param provider: Ingest Provider Details.
                .. seealso:: :class: `superdesk.io.ingest_provider_model.IngestProviderResource`
        :type provider: dict
        :param error:
        :type error: :py:class: `superdes.errors.SuperdeskIngestError`
        """

        if provider.get('critical_errors', {}).get(str(error.code)):
            updates = {
                'is_closed': True,
                'last_closed': {
                    'closed_at': utcnow(),
                    'message': 'Channel closed due to critical error: {}'.format(error)
                }
            }

            get_resource_service('ingest_providers').system_update(provider[superdesk.config.ID_FIELD],
                                                                   updates, provider)

    def add_timestamps(self, item):
        """
        Adds firstcreated and versioncreated timestamps to item

        :param item: object which can be saved to ingest collection
        :type item: dict
        """

        item['firstcreated'] = utc.localize(item['firstcreated']) if item.get('firstcreated') else utcnow()
        item['versioncreated'] = utc.localize(item['versioncreated']) if item.get('versioncreated') else utcnow()

    def log_item_error(self, err, item, provider):
        """TODO: put item into provider error basket."""
        logger.warning('ingest error msg={} item={} provider={}'.format(
            str(err),
            item.get('guid'),
            provider.get('name')
        ))

    def prepare_href(self, href):
        """
        Prepare a link to an external resource (e.g. an image file) so that it can be directly used by the ingest
        machinery for fetching it.

        Sub-classes can override this method if properties like HTTP Authentication need to be added to the href.

        :param href: the original URL as extracted from an RSS entry
        :type href: str
        :return: prepared URL
        :rtype: str
        """
        return href

    def get_feed_parser(self, provider, article=None):
        """
        Returns instance of configured feed parser for the given provider.

        :param provider: Ingest Provider Details.
        :type provider: dict :py:class: `superdesk.io.ingest_provider_model.IngestProviderResource`
        :param article: article which needs to parsed by the parser, defaults to None. When None, the registered parser
                        will be returned without checking if the parser can parse the article.
        :return: Feed Parser instance.
        :rtype: :py:class:`superdesk.io.feed_parsers.FeedParser`
        :raises: SuperdeskIngestError.parserNotFoundError()
                    if either feed_parser value is empty or Feed Parser not found.
        """

        parser = registered_feed_parsers.get(provider.get('feed_parser', ''))
        if not parser:
            raise SuperdeskIngestError.parserNotFoundError(provider=provider)

        if article and not parser.can_parse(article):
            raise SuperdeskIngestError.parserNotFoundError(provider=provider)

        if article:
            parser = parser.__class__()

        return parser


# must be imported for registration
import superdesk.io.feeding_services.email  # NOQA
import superdesk.io.feeding_services.file_service  # NOQA
import superdesk.io.feeding_services.ftp  # NOQA
import superdesk.io.feeding_services.http_service  # NOQA
import superdesk.io.feeding_services.rss  # NOQA



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.locators.locators import find_cities
import logging
from apps.archive.common import format_dateline_to_locmmmddsrc
from superdesk.utc import get_date
import superdesk
from superdesk.metadata.item import CONTENT_TYPE
from apps.publish.content.common import ITEM_PUBLISH

logger = logging.getLogger(__name__)


def update_to_pass_validation(item, **kwargs):
    """
    This is a test macro that does what is required to ensure that a text item will pass publication validation.
    It is intended to be used to test auto publishing, that is publishing directly from ingest.
    At the moment virtually all content received from Reuters fails validation.
    :param item:
    :param kwargs:
    :return:
    """
    try:
        lookup = {'act': ITEM_PUBLISH, 'type': CONTENT_TYPE.TEXT}
        validators = superdesk.get_resource_service('validators').get(req=None, lookup=lookup)
        if validators.count():
            max_slugline_len = validators[0]['schema']['slugline']['maxlength']
            max_headline_len = validators[0]['schema']['headline']['maxlength']
            item['slugline'] = item['slugline'][:max_slugline_len] \
                if len(item['slugline']) > max_slugline_len else item['slugline']
            item['headline'] = item['headline'][:max_headline_len] \
                if len(item['headline']) > max_headline_len else item['headline']
        if 'dateline' not in item:
            cities = find_cities(country_code='AU', state_code='NSW')
            located = [c for c in cities if c['city'].lower() == 'sydney']
            if located:
                item['dateline'] = {'date': item['firstcreated'], 'located': located[0]}
            item['dateline']['source'] = item['source']
            item['dateline']['text'] = format_dateline_to_locmmmddsrc(located[0], get_date(item['firstcreated']),
                                                                      source=item['source'])
        return item
    except:
        logging.exception('Test update to pass validation macro exception')


name = 'update to pass validation'
label = 'Update to pass validation'
shortcut = '$'
callback = update_to_pass_validation



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging

from superdesk.metadata.item import ITEM_TYPE, CONTENT_TYPE
from superdesk.metadata.utils import is_takes_package

formatters = []
logger = logging.getLogger(__name__)


class FormatterRegistry(type):
    """Registry metaclass for formatters."""

    def __init__(cls, name, bases, attrs):
        """Register sub-classes of Formatter class when defined."""
        super(FormatterRegistry, cls).__init__(name, bases, attrs)
        if name != 'Formatter':
            formatters.append(cls())


class Formatter(metaclass=FormatterRegistry):
    """Base Formatter class for all types of Formatters like News ML 1.2, News ML G2, NITF, etc."""

    def format(self, article, subscriber):
        """Formats the article and returns the transformed string"""
        raise NotImplementedError()

    def can_format(self, format_type, article):
        """Test if formatter can format for given article."""
        raise NotImplementedError()

    def append_body_footer(self, article):
        """
        Checks if the article has any Public Service Announcements and if available appends each of them to the body.

        :return: body with public service announcements.
        """

        body = ''
        if article[ITEM_TYPE] in [CONTENT_TYPE.TEXT, CONTENT_TYPE.PREFORMATTED] or is_takes_package(article):
            body = article.get('body_html', '')
        elif article[ITEM_TYPE] in [CONTENT_TYPE.AUDIO, CONTENT_TYPE.PICTURE, CONTENT_TYPE.VIDEO]:
            body = article.get('description', '')

        if body and article.get('body_footer'):
            body = '{}<br>{}'.format(body, article.get('body_footer'))

        return body

    def append_legal(self, article, truncate=False):
        """
        Checks if the article has the legal flag on and adds 'Legal:' to the slugline
        :param article: article having the slugline
        :param truncate: truncates the slugline to 24 characters
        :return: updated slugline
        """
        slugline = article.get('slugline', '') or ''

        if article.get('flags', {}).get('marked_for_legal', False):
            slugline = '{}: {}'.format('Legal', slugline)
            if truncate:
                slugline = slugline[:24]

        return slugline


def get_formatter(format_type, article):
    """Get parser for given xml.

    :param etree: parsed xml
    """
    for formatter in formatters:
        if formatter.can_format(format_type, article):
            return formatter


import superdesk.publish.formatters.nitf_formatter  # NOQA
import superdesk.publish.formatters.ninjs_formatter  # NOQA
import superdesk.publish.formatters.newsml_1_2_formatter  # NOQA



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import os
import pymongo
import unittest
import elasticsearch
import logging

from base64 import b64encode
from flask import json, Config
from superdesk.notification_mock import setup_notification_mock, teardown_notification_mock
from superdesk import get_resource_service
from superdesk.factory import get_app
from eve_elastic import get_es, get_indices

test_user = {
    'username': 'test_user',
    'password': 'test_password',
    'is_active': True,
    'is_enabled': True,
    'needs_activation': False,
    'sign_off': 'abc',
    'email': 'behave_test@sourcefabric.org',
    'preferences': {
        'email:notification': {
            'label': 'Send notifications via email',
            'type': 'bool',
            'default': True,
            'category': 'notifications',
            'enabled': True}
    }
}


def get_mongo_uri(key, dbname):
    """Read mongo uri from env variable and replace dbname.

    :param key: env variable name
    :param dbname: mongo db name to use
    """
    env_uri = os.environ.get(key, 'mongodb://localhost/test')
    env_host = env_uri.rsplit('/', 1)[0]
    return '/'.join([env_host, dbname])


def get_test_settings():
    test_settings = {}
    test_settings['ELASTICSEARCH_INDEX'] = 'sptest'
    test_settings['MONGO_URI'] = get_mongo_uri('MONGO_URI', 'sptests')
    test_settings['LEGAL_ARCHIVE_DBNAME'] = 'sptests_legal_archive'
    test_settings['LEGAL_ARCHIVE_URI'] = get_mongo_uri('LEGAL_ARCHIVE_URI', 'sptests_legal_archive')
    test_settings['ARCHIVED_DBNAME'] = 'sptests_archived'
    test_settings['ARCHIVED_URI'] = get_mongo_uri('ARCHIVED_URI', 'sptests_archived')
    test_settings['DEBUG'] = True
    test_settings['TESTING'] = True
    test_settings['SUPERDESK_TESTING'] = True
    test_settings['BCRYPT_GENSALT_WORK_FACTOR'] = 4
    test_settings['CELERY_ALWAYS_EAGER'] = 'True'
    test_settings['CONTENT_EXPIRY_MINUTES'] = 99
    test_settings['VERSION'] = '_current_version'
    return test_settings


def drop_elastic(app):
    with app.app_context():
        try:
            es = get_es(app.config['ELASTICSEARCH_URL'])
            get_indices(es).delete(app.config['ELASTICSEARCH_INDEX'])
        except elasticsearch.exceptions.NotFoundError:
            pass


def drop_mongo(app):
    with app.app_context():
        drop_mongo_db(app, 'MONGO', 'MONGO_DBNAME')
        drop_mongo_db(app, 'LEGAL_ARCHIVE', 'LEGAL_ARCHIVE_DBNAME')
        drop_mongo_db(app, 'ARCHIVED', 'ARCHIVED_DBNAME')


def drop_mongo_db(app, db_prefix, dbname):
    try:
        if app.config.get(dbname):
            app.data.mongo.pymongo(prefix=db_prefix).cx.drop_database(app.config[dbname])
    except pymongo.errors.ConnectionFailure:
        raise ValueError('Invalid mongo config or server is down (uri=%s db=%s)' %
                         (db_prefix, app.config[dbname]))
    except AttributeError:
        pass


def setup(context=None, config=None, app_factory=get_app):

    app_abspath = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
    app_config = Config(app_abspath)
    app_config.from_object('test_settings')
    app_config['APP_ABSPATH'] = app_abspath

    app_config.update(get_test_settings())
    if config:
        app_config.update(config)

    app = app_factory(app_config)
    logger = logging.getLogger('superdesk')
    logger.setLevel(logging.ERROR)
    logger = logging.getLogger('elasticsearch')
    logger.setLevel(logging.ERROR)
    logger = logging.getLogger('urllib3')
    logger.setLevel(logging.ERROR)
    drop_elastic(app)
    drop_mongo(app)

    # create index again after dropping it
    app.data.elastic.init_app(app)

    if context:
        context.app = app
        context.client = app.test_client()


def setup_auth_user(context, user=None):
    setup_db_user(context, user)


def add_to_context(context, token, user):
    context.headers.append(('Authorization', b'basic ' + b64encode(token + b':')))
    context.user = user
    set_placeholder(context, 'CONTEXT_USER_ID', str(user.get('_id')))


def set_placeholder(context, name, value):
    old_p = getattr(context, 'placeholders', None)
    if not old_p:
        context.placeholders = dict()
    context.placeholders[name] = value


def get_prefixed_url(current_app, endpoint):
    if endpoint.startswith('http://'):
        return endpoint

    endpoint = endpoint if endpoint.startswith('/') else ('/' + endpoint)
    url = current_app.config['URL_PREFIX'] + endpoint
    return url


def setup_db_user(context, user):
    """
    Setup the user for the DB authentication.
    :param context: test context
    :param dict user: user
    """
    user = user or test_user
    with context.app.test_request_context(context.app.config['URL_PREFIX']):
        original_password = user['password']

        user.setdefault('user_type', 'administrator')

        if not get_resource_service('users').find_one(username=user['username'], req=None):
            get_resource_service('users').post([user])

        user['password'] = original_password
        auth_data = json.dumps({'username': user['username'], 'password': user['password']})
        auth_response = context.client.post(get_prefixed_url(context.app, '/auth'),
                                            data=auth_data, headers=context.headers)

        token = json.loads(auth_response.get_data()).get('token').encode('ascii')
        add_to_context(context, token, user)


def setup_notification(context):
    setup_notification_mock(context)


def teardown_notification(context):
    teardown_notification_mock(context)


class TestCase(unittest.TestCase):

    def setUp(self):
        setup(self, app_factory=get_app)
        self.ctx = self.app.app_context()
        self.ctx.push()

    def tearDown(self):
        if hasattr(self, 'ctx'):
            self.ctx.pop()

    def get_fixture_path(self, filename):
        rootpath = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
        return os.path.join(rootpath, 'features', 'steps', 'fixtures', filename)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import arrow
import datetime
from pytz import utc, timezone  # flake8: noqa

tzinfo = getattr(datetime, 'tzinfo', object)


def utcnow():
    """Get tz aware datetime object.

    Remove microseconds which can't be persisted by mongo so we have
    the values consistent in both mongo and elastic.
    """
    if hasattr(datetime.datetime, 'now'):
        now = datetime.datetime.now(tz=utc)
    else:
        now = datetime.datetime.utcnow()
    return now.replace(microsecond=0)


def get_date(date_or_string):
    if date_or_string:
        return arrow.get(date_or_string).datetime


def get_expiry_date(minutes, offset=None):
    if offset:
        if type(offset) is not datetime:
            return offset + datetime.timedelta(minutes=minutes)
        else:
            raise TypeError('offset must be a datetime.date, not a %s' % type(offset))
    else:
        return utcnow() + datetime.timedelta(minutes=minutes)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging

from eve.utils import config
from flask import json

import superdesk
from apps.archive.archive import SOURCE as ARCHIVE
from apps.archive.common import generate_unique_id_and_name
from apps.archive.common import insert_into_versions, remove_unwanted, set_original_creator
from apps.tasks import send_to
from superdesk import get_resource_service
from superdesk.errors import SuperdeskApiError, ProviderError
from superdesk.metadata.item import GUID_TAG, FAMILY_ID, INGEST_ID, ITEM_STATE, CONTENT_STATE
from superdesk.metadata.utils import generate_guid
from apps.search_providers.aap_mm import PROVIDER_NAME
from superdesk.utc import utcnow

logger = logging.getLogger(__name__)


class AAPMMService(superdesk.Service):

    def create(self, docs, **kwargs):
        search_provider = get_resource_service('search_providers').find_one(search_provider=PROVIDER_NAME, req=None)

        if not search_provider or search_provider.get('is_closed', False):
            raise SuperdeskApiError.badRequestError('No search provider found or the search provider is closed.')

        if 'config' in search_provider:
            self.backend.set_credentials(search_provider['config'])

        new_guids = []
        for doc in docs:
            if not doc.get('desk'):  # if no desk is selected then it is bad request
                raise SuperdeskApiError.badRequestError("Destination desk cannot be empty.")

            try:
                archived_doc = self.backend.find_one_raw(doc['guid'], doc['guid'])
            except FileNotFoundError as ex:
                raise ProviderError.externalProviderError(ex, search_provider)

            dest_doc = dict(archived_doc)
            new_id = generate_guid(type=GUID_TAG)
            new_guids.append(new_id)
            dest_doc[config.ID_FIELD] = new_id
            generate_unique_id_and_name(dest_doc)

            if search_provider:
                dest_doc['ingest_provider'] = str(search_provider[config.ID_FIELD])

            dest_doc[config.VERSION] = 1
            send_to(doc=dest_doc, update=None, desk_id=doc.get('desk'), stage_id=doc.get('stage'))
            dest_doc[ITEM_STATE] = doc.get(ITEM_STATE, CONTENT_STATE.FETCHED)
            dest_doc[INGEST_ID] = archived_doc[config.ID_FIELD]
            dest_doc[FAMILY_ID] = archived_doc[config.ID_FIELD]
            remove_unwanted(dest_doc)
            set_original_creator(dest_doc)

            superdesk.get_resource_service(ARCHIVE).post([dest_doc])
            insert_into_versions(dest_doc[config.ID_FIELD])

            get_resource_service('search_providers').system_update(search_provider[config.ID_FIELD],
                                                                   {'last_item_update': utcnow()}, search_provider)

        return new_guids

    def get(self, req, lookup):
        search_provider = get_resource_service('search_providers').find_one(search_provider=PROVIDER_NAME, req=None)

        if not search_provider or search_provider.get('is_closed', False):
            raise SuperdeskApiError.badRequestError('No search provider found or the search provider is closed.')

        if search_provider:
            if 'config' in search_provider:
                self.backend.set_credentials(search_provider['config'])

            query = self._get_query(req)
            results = self.backend.find(PROVIDER_NAME, query, None)

            for doc in results.docs:
                doc['ingest_provider'] = str(search_provider[config.ID_FIELD])

            return results

    def _get_query(self, req):
        args = getattr(req, 'args', {})
        query = json.loads(args.get('source')) if args.get('source') else {'query': {'filtered': {}}}

        return query



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
import superdesk
from superdesk.io.iptc import subject_codes
from superdesk.locators.locators import find_cities
from apps.archive.common import format_dateline_to_locmmmddsrc
from superdesk.utc import get_date

logger = logging.getLogger(__name__)


def noise11_derive_metadata(item, **kwargs):
    """
    By definition anyhting from NOISE11 will be entertainment so set the category, subject and dateline
    appropriately
    :param item:
    :param kwargs:
    :return:
    """
    try:
        if 'anpa_category' not in item:
            category_map = superdesk.get_resource_service('vocabularies').find_one(req=None, _id='categories')
            if category_map:
                map_entry = next((code for code in category_map['items'] if code['qcode'] == 'e' and code['is_active']),
                                 None)
                item['anpa_category'] = [{'qcode': 'e', 'name': map_entry['name']}]

        if 'subject' not in item:
            qcode = '01000000'
            item['subject'] = [{'qcode': qcode, 'name': subject_codes[qcode]}]

        cities = find_cities(country_code='AU', state_code='NSW')
        located = [c for c in cities if c['city'].lower() == 'sydney']

        if located and 'dateline' not in item:
            item['dateline'] = {'date': item['firstcreated'], 'located': located[0]}
        item['dateline']['source'] = item['source']
        item['dateline']['text'] = format_dateline_to_locmmmddsrc(located[0], get_date(item['firstcreated']),
                                                                  source=item['source'])

        return item
    except Exception as ex:
        logger.exception(ex)


name = 'Derive metadata for Noise11'
label = 'Noise11 metadata derivation'
shortcut = '$'
callback = noise11_derive_metadata



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import re
import logging
import collections
from flask import json
from superdesk.services import BaseService
from superdesk.errors import SuperdeskApiError
from apps.dictionaries.resource import DICTIONARY_FILE


logger = logging.getLogger(__name__)


def encode_dict(words_dict):
    return json.dumps(words_dict)


def decode_dict(words_list):
    return json.loads(words_list)


def train(features):
    model = collections.defaultdict(lambda: 1)
    for f in features:
        model[f] += 1
    return model


def words(text):
    return [w for w in re.findall('[^\W_]+', text.lower()) if not w.isdigit()]


def add_word(words, word, count):
    """Add word with count to words model.

    :param words: words model
    :param word
    :param count
    """
    words.setdefault(word, 0)
    words[word] += count


def add_words(nwords, text, val=1):
    for word in words(text):
        add_word(nwords, word, val)


def read(stream):
    return stream.read().decode('utf-8')


def merge(doc, words):
    doc.setdefault('content', {})
    for word, count in words.items():
        add_word(doc['content'], word, count)


def read_from_file(doc):
    content = doc.pop(DICTIONARY_FILE)
    if 'text/' not in content.mimetype:
        raise SuperdeskApiError.badRequestError('A text dictionary file is required')
    return train(words(read(content)))


class DictionaryService(BaseService):
    def on_create(self, docs):
        for doc in docs:
            if self.find_one(req=None, name=doc['name'], language_id=doc['language_id']):
                raise SuperdeskApiError.badRequestError(message='The dictionary already exists',
                                                        payload={'name': 'duplicate'})
            if doc.get(DICTIONARY_FILE):
                words = read_from_file(doc)
                merge(doc, words)
            if 'content' in doc:
                doc['content'] = encode_dict(doc['content'])

    def on_created(self, docs):
        for doc in docs:
            doc.pop('content', None)

    def on_updated(self, updates, original):
        if 'content' in updates:
            updates['content'] = decode_dict(updates['content'])

    def find_one(self, req, **lookup):
        doc = super().find_one(req, **lookup)
        if doc and 'content' in doc:
            doc['content'] = decode_dict(doc['content'])
        return doc

    def on_fetched(self, result):
        for doc in result['_items']:
            if 'content' in doc:
                doc['content'] = decode_dict(doc['content'])

    def get_base_language(self, lang):
        if lang and lang.find('-') > 0:
            return lang.split('-')[0]

    def get_dictionaries(self, lang):
        """
        Returns all the active dictionaries. If both the language (en-AU)
        and the base language (en) available, it will return the dict with language
        :param lang:
        :return:
        """
        languages = [{'language_id': lang}]
        base_language = self.get_base_language(lang)
        if base_language:
            languages.append({'language_id': base_language})

        lookup = {'$and': [{'$or': languages},
                           {'$or': [{'is_active': {'$exists': 0}}, {'is_active': 'true'}]}]}
        dicts = list(self.get(req=None, lookup=lookup))
        langs = [d['language_id'] for d in dicts]

        if base_language and base_language in langs and lang in langs:
            dicts = [d for d in dicts if d['language_id'] != base_language]

        return dicts

    def get_model_for_lang(self, lang):
        """Get model for given language.

        It will use all active dictionaries for given language combined.

        :param lang: language code
        """
        model = {}
        dicts = self.get_dictionaries(lang)

        for _dict in dicts:
            if 'content' in _dict:
                content = decode_dict(_dict['content'])
            else:
                content = {}
            for word, count in content.items():
                add_word(model, word, count)
        return model

    def on_update(self, updates, original):
        # parse json list
        if updates.get('content_list'):
            updates['content'] = json.loads(updates.pop('content_list'))

        # handle manual changes
        nwords = original.get('content', {}).copy()
        for word, val in updates.get('content', {}).items():
            if val:
                add_words(nwords, word, val)
            else:
                nwords.pop(word, None)
        updates['content'] = nwords

        # handle uploaded file
        if updates.get(DICTIONARY_FILE):
            file_words = read_from_file(updates)
            merge(updates, file_words)

        # save it as string, otherwise it would order keys and it takes forever
        if 'content' in updates:
            updates['content'] = encode_dict(updates['content'])




from test_factory import SuperdeskTestCase
import superdesk
from eve.utils import ParsedRequest

from apps.dictionaries.service import words, DictionaryService


class WordsTestCase(SuperdeskTestCase):

    def setUp(self):
        super().setUp()
        self.req = ParsedRequest()
        with self.app.test_request_context(self.app.config.get('URL_PREFIX')):
            self.dictionaries = [{'_id': '1', 'name': 'Eng', 'language_id': 'en'},
                                 {'_id': '2', 'name': 'Eng AUs', 'language_id': 'en-AU', 'is_active': 'true'},
                                 {'_id': '3', 'name': 'French', 'language_id': 'fr'}]
            self.app.data.insert('dictionaries', self.dictionaries)

    def test_words_parsing(self):
        self.assertEquals(['abc'], words('abc'))
        self.assertEquals(['3d', 'x5'], words('3d 123 x5'))

    def test_base_language(self):
        self.assertEqual(DictionaryService().get_base_language('en-AU'), 'en')
        self.assertIsNone(DictionaryService().get_base_language('en'))

    def test_get_dictionary(self):
        with self.app.app_context():
            dicts = superdesk.get_resource_service('dictionaries').get_dictionaries('en')
            self.assertEqual(len(dicts), 1)
            self.assertEqual(dicts[0]['language_id'], 'en')
            dicts = superdesk.get_resource_service('dictionaries').get_dictionaries('en-AU')
            self.assertEqual(len(dicts), 1)
            self.assertEqual(dicts[0]['language_id'], 'en-AU')



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from unittest import TestCase
from .reuters_derive_dateline import reuters_derive_dateline
import datetime


class DPADeriveDatelineTests(TestCase):
    def simple_case_test(self):
        item = dict()
        item['firstcreated'] = datetime.datetime(2015, 10, 26, 11, 45, 19, 0)
        item['body_html'] = '<p>DETROIT (Reuters) - General Motors Co <GM.N> Chief Financial Officer Chuck Stevens \
            said on Wednesday the macroeconomic challenges in Brazil will remain in the near term but the company \
            has \"huge upside leverage once the macro situation changes\" in South America\'s largest \
            economy.</p>\n<p>GM\'s car sales so far in October are up versus a year ago, Stevens said to reporters \
            after the No. 1 U.S. automaker reported third-quarter financial results.</p>\n<p>Stevens also \
            reaffirmed GM\'s past forecasts that it will show profit in Europe in 2016. It would be GM\'s first \
            profit in Europe since 1999.</p>\n<p> (Reporting by Bernie Woodall and Joseph White; \
            Editing by Chizu Nomiyamam and Jeffrey Benkoe)</p>'
        reuters_derive_dateline(item)
        self.assertEqual(item['dateline']['located']['city'], 'Detroit')

    def with_a_date_test(self):
        item = dict()
        item['firstcreated'] = datetime.datetime(2015, 10, 26, 11, 45, 19, 0)
        item['body_html'] = '<p>PARIS, Oct 22 (Reuters) - Eurotunnel said on\nThursday that third-quarter revenue \
        rose 3 percent to 334.4\nmillion euros ($379.48 million), as economic recovery helped\noffset the \
        impact of the disruption to traffic resulting from\nthe migrant crisis.</p>\n<p>The operator of the \
        Channel Tunnel linking France and\nBritain said that business remained dynamic, driven by a\nrecovering \
        economy in Britain and to a lesser extent in the\neuro-zone.</p>\n<p>But a camp of around 6,000 migrants \
        in the Calais area\nfleeing war, political turmoil and poverty outside Europe has\ncaused disruption to \
        traffic since Summer.</p>\n<p>Eurotunnel carries Eurostar high-speed trains between Paris,\nBrussels and \
        London, as well as shuttle trains containing\npassenger cars, coaches and freight trucks.</p>\n<p>Rail \
        freight tonnage fell 27 percent year-on-year and the\nnumber of freight trains using the Channel \
        tunnel fell 33\npercent, the company said - blaming the drop on the migrant\ncrisis.</p>\n<p>Passenger \
        traffic in the quarter rose 2 percent year-on-year\nto 2,866,155 on the Eurostar. Traffic however fell \
        1 percent on\ntrucks and 9 percent on coaches compared with the same period\nlast year.</p>\n<p>In July \
        Eurotunnel asked the French and British governments\nto reimburse it for close to 10 million euros it spent \
        to beef\nup security to cope with a migrant crisis at the French port of\nCalais.</p>\n<p>Third quarter \
        sales figures no longer include MyFerryLink,\nthe ferry service between Britain and France, which ended \
        its \nits activity on June 29.</p>\n<p>($1 = 0.8812 euros)\n\n (Reporting by Dominique Vidalon; Editing \
        by Andrew Callus)</p>",'
        reuters_derive_dateline(item)
        self.assertEqual(item['dateline']['located']['city'], 'PARIS')

    def with_a_byline_test(self):
        item = dict()
        item['firstcreated'] = datetime.datetime(2015, 10, 26, 11, 45, 19, 0)
        item['byline'] = 'By Karl Plume'
        item['body_html'] = '<p>By Karl Plume</p>\n<p>CHICAGO, Oct 21 (Reuters) - Chicago Cubs supporters have \
        uttered the phrase \"wait till next year\" perhaps more than any other fans in baseball, with their team\'s \
        championship drought stretching to 107 years after being swept from this year\'s playoffs by the New York \
        Mets.</p>\n<p>However, the 2015 Cubs have given Chicago\'s north-side faithful reason to believe that \
        their wait for a title, the longest in U.S. professional sports, might soon come to an end.</p>\n<p>The \
        Mets ensured the Cubs\' unprecedented streak will continue another year with an 8-3 victory on Wednesday \
        that saw them capture the National League pennant and claim a place in the World Series against \
        Kansas City or Toronto.</p>\n<p>While the Cubs\' clubhouse was disappointed after the defeat, there were \
        real signs of hope.</p>'
        reuters_derive_dateline(item)
        self.assertEqual(item['dateline']['located']['city'], 'Chicago')

    def with_a_dateline_already_leave_it_alone_test(self):
        item = dict()
        item['firstcreated'] = datetime.datetime(2015, 10, 26, 11, 45, 19, 0)
        item['dateline'] = {'located': {'city': 'Chicargo'}}
        item['body_html'] = '<p>DONT CARE (Reuters) - Chicago Cubs supporters have \
        uttered the phrase \"wait till next year\" perhaps more than any other fans in baseball, with their team\'s \
        championship drought stretching to 107 years after being swept from this year\'s playoffs by the New York \
        Mets.</p>\n<p>However, the 2015 Cubs have given Chicago\'s north-side faithful reason to believe that \
        their wait for a title, the longest in U.S. professional sports, might soon come to an end.</p>\n<p>The \
        Mets ensured the Cubs\' unprecedented streak will continue another year with an 8-3 victory on Wednesday \
        that saw them capture the National League pennant and claim a place in the World Series against \
        Kansas City or Toronto.</p>\n<p>While the Cubs\' clubhouse was disappointed after the defeat, there were \
        real signs of hope.</p>'
        reuters_derive_dateline(item)
        self.assertEqual(item['dateline']['located']['city'], 'Chicargo')

    def with_just_a_date_test(self):
        item = dict()
        item['firstcreated'] = datetime.datetime(2015, 10, 26, 11, 45, 19, 0)
        item['body_html'] = '<p>Oct 22 (Reuters) - Eurotunnel said on\nThursday that third-quarter revenue \
        rose 3 percent to 334.4\nmillion euros ($379.48 million), as economic recovery helped\noffset the \
        impact of the disruption to traffic resulting from\nthe migrant crisis.</p>\n<p>The operator of the \
        Channel Tunnel linking France and\nBritain said that business remained dynamic, driven by a\nrecovering \
        economy in Britain and to a lesser extent in the\neuro-zone.</p>'
        reuters_derive_dateline(item)
        self.assertNotIn('dateline', item)

    def from_bagalore_test(self):
        item = {'dateline': {'located': {'city': 'Bangalore'}}}
        item['firstcreated'] = datetime.datetime(2015, 10, 26, 11, 45, 19, 0)
        item['body_html'] = '<p>Wagga Wagga (Reuters) - Chicago Cubs supporters have \
        uttered the phrase \"wait till next year\" perhaps more than any other fans in baseball, with their team\'s \
        championship drought stretching to 107 years after being swept from this year\'s playoffs by the New York \
        Mets.</p>\n<p>However, the 2015 Cubs have given Chicago\'s north-side faithful reason to believe that \
        their wait for a title, the longest in U.S. professional sports, might soon come to an end.</p>\n<p>The \
        Mets ensured the Cubs\' unprecedented streak will continue another year with an 8-3 victory on Wednesday \
        that saw them capture the National League pennant and claim a place in the World Series against \
        Kansas City or Toronto.</p>\n<p>While the Cubs\' clubhouse was disappointed after the defeat, there were \
        real signs of hope.</p>'
        reuters_derive_dateline(item)
        self.assertEqual(item['dateline']['located']['city'], 'Wagga Wagga')



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import requests
import re

RATE_SERVICE = 'http://download.finance.yahoo.com/d/quotes.csv?s={}=X&f=nl1d1'


def get_rate(from_currency, to_currency):
    """Get the exchange rate."""
    r = requests.get(RATE_SERVICE.format(from_currency + to_currency), timeout=5)
    return float(r.text.split(',')[1])


def do_conversion(item, rate, prefix, search_param):
    """
    Performs the conversion
    :param item: story
    :param rate: exchange rate
    :param prefix: to be used in the results
    :param search_param: search parameter to locate the original value.  It should
    be a valid regular expression pattern, and not just an arbitrary string.
    :return: modified story
    """

    def convert(match):
        from_value = float(match.group(1))
        to_value = rate * from_value
        return prefix % to_value

    for field in ['body_html', 'body_text', 'abstract', 'headline', 'slugline']:
        if item.get(field, None):
            item[field] = re.sub(search_param, convert, item[field])

    return item



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.publish.formatters import Formatter
from apps.publish.formatters.aap_formatter_common import map_priority
import superdesk
from superdesk.errors import FormatterError
from bs4 import BeautifulSoup
import datetime
from superdesk.metadata.item import ITEM_TYPE, CONTENT_TYPE, BYLINE, EMBARGO
from .field_mappers.selectorcode_mapper import SelectorcodeMapper
from .field_mappers.locator_mapper import LocatorMapper


class AAPAnpaFormatter(Formatter):
    def format(self, article, subscriber):
        try:
            docs = []
            for category in article.get('anpa_category'):
                pub_seq_num = superdesk.get_resource_service('subscribers').generate_sequence_number(subscriber)
                anpa = []

                # selector codes are only injected for those subscribers that are defined
                # in the mapper
                selectors = dict()
                SelectorcodeMapper().map(article, category.get('qcode').upper(),
                                         subscriber=subscriber,
                                         formatted_item=selectors)
                if 'selector_codes' in selectors and selectors['selector_codes']:
                    anpa.append(b'\x05')
                    anpa.append(selectors['selector_codes'].encode('ascii'))
                    anpa.append(b'\x0D\x0A')

                # start of message header (syn syn soh)
                anpa.append(b'\x16\x16\x01')
                anpa.append(article.get('service_level', 'a').lower().encode('ascii'))

                # story number
                anpa.append(str(pub_seq_num).zfill(4).encode('ascii'))

                # field seperator
                anpa.append(b'\x0A')  # -LF
                anpa.append(map_priority(article.get('priority')).encode('ascii'))
                anpa.append(b'\x20')

                anpa.append(category['qcode'].encode('ascii'))

                anpa.append(b'\x13')
                # format identifier
                if article[ITEM_TYPE] == CONTENT_TYPE.PREFORMATTED:
                    anpa.append(b'\x12')
                else:
                    anpa.append(b'\x11')
                anpa.append(b'\x20')

                # keyword
                keyword = 'bc-{}'.format(self.append_legal(article=article, truncate=True)).replace(' ', '-')
                keyword = keyword[:24] if len(keyword) > 24 else keyword
                anpa.append(keyword.encode('ascii'))
                anpa.append(b'\x20')

                # version field
                anpa.append(b'\x20')

                # reference field
                anpa.append(b'\x20')

                # filing date
                anpa.append('{}-{}'.format(article['_updated'].strftime('%m'),
                                           article['_updated'].strftime('%d')).encode('ascii'))
                anpa.append(b'\x20')

                # add the word count
                anpa.append(str(article.get('word_count', '0000')).zfill(4).encode('ascii'))
                anpa.append(b'\x0D\x0A')

                anpa.append(b'\x02')  # STX

                self._process_headline(anpa, article, category['qcode'].encode('ascii'))

                keyword = self.append_legal(article=article, truncate=True).encode('ascii', 'ignore')
                anpa.append(keyword)
                take_key = article.get('anpa_take_key', '').encode('ascii', 'ignore')
                anpa.append((b'\x20' + take_key) if len(take_key) > 0 else b'')
                anpa.append(b'\x0D\x0A')

                if BYLINE in article:
                    anpa.append(article.get(BYLINE).encode('ascii', 'ignore'))
                    anpa.append(b'\x0D\x0A')

                if 'dateline' in article and 'text' in article['dateline']:
                    anpa.append(article.get('dateline').get('text').encode('ascii', 'ignore'))

                body = self.append_body_footer(article)
                if article.get(EMBARGO):
                    embargo = '{}{}'.format('Embargo Content. Timestamp: ', article.get(EMBARGO).isoformat())
                    body = embargo + body

                if article[ITEM_TYPE] == CONTENT_TYPE.PREFORMATTED:
                    anpa.append(body.encode('ascii', 'replace'))
                else:
                    anpa.append(BeautifulSoup(body, "html.parser").text.encode('ascii', 'replace'))

                anpa.append(b'\x0D\x0A')
                if article.get('more_coming', False):
                    anpa.append('MORE'.encode('ascii'))
                else:
                    anpa.append(article.get('source', '').encode('ascii'))
                sign_off = article.get('sign_off', '').encode('ascii')
                anpa.append((b'\x20' + sign_off) if len(sign_off) > 0 else b'')
                anpa.append(b'\x0D\x0A')

                anpa.append(b'\x03')  # ETX

                # time and date
                anpa.append(datetime.datetime.now().strftime('%d-%m-%y %H-%M-%S').encode('ascii'))

                anpa.append(b'\x04')  # EOT
                anpa.append(b'\x0D\x0A\x0D\x0A\x0D\x0A\x0D\x0A\x0D\x0A\x0D\x0A\x0D\x0A\x0D\x0A')

                docs.append((pub_seq_num, b''.join(anpa)))

            return docs
        except Exception as ex:
            raise FormatterError.AnpaFormatterError(ex, subscriber)

    def _process_headline(self, anpa, article, category):
        # prepend the locator to the headline if required
        headline_prefix = LocatorMapper().map(article, category.upper())
        if headline_prefix:
            headline = '{}:{}'.format(headline_prefix, article['headline'])
        else:
            headline = article.get('headline', '')

        # Set the maximum size to 64 including the sequence number if any
        if len(headline) > 64:
            if article.get('sequence'):
                digits = len(str(article['sequence'])) + 1
                shortened_headline = '{}={}'.format(headline[:-digits][:(64 - digits)], article['sequence'])
                anpa.append(shortened_headline.encode('ascii', 'replace'))
            else:
                anpa.append(headline[:64].encode('ascii', 'replace'))
        else:
            anpa.append(headline.encode('ascii', 'replace'))
        anpa.append(b'\x0D\x0A')

    def can_format(self, format_type, article):
        return format_type == 'AAP ANPA' and article[ITEM_TYPE] in [CONTENT_TYPE.TEXT, CONTENT_TYPE.PREFORMATTED]



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from copy import copy
from functools import partial
import logging
from copy import deepcopy

from eve.versioning import resolve_document_version
from eve.utils import config, ParsedRequest
from eve.validation import ValidationError

import superdesk
from apps.content import push_content_notification
from superdesk.metadata.item import PUB_STATUS, CONTENT_TYPE, ITEM_TYPE, GUID_FIELD, ITEM_STATE, CONTENT_STATE, \
    PUBLISH_STATES, EMBARGO
from superdesk.metadata.packages import SEQUENCE, LINKED_IN_PACKAGES, GROUPS, PACKAGE
from superdesk.publish import SUBSCRIBER_TYPES
from superdesk.errors import InvalidStateTransitionError, SuperdeskApiError, PublishQueueError
from superdesk.notification import push_notification
from superdesk.services import BaseService
from superdesk import get_resource_service
from apps.archive.archive import ArchiveResource, SOURCE as ARCHIVE
from apps.archive.common import validate_schedule, ITEM_OPERATION, convert_task_attributes_to_objectId, is_genre, \
    BROADCAST_GENRE, get_expiry
from superdesk.utc import utcnow
from superdesk.workflow import is_workflow_state_transition_valid
from superdesk.publish.formatters import get_formatter
from apps.common.components.utils import get_component
from apps.item_autosave.components.item_autosave import ItemAutosave
from superdesk.metadata.utils import item_url
from apps.archive.common import get_user, insert_into_versions, item_operations
from apps.packages import TakesPackageService
from apps.packages.package_service import PackageService
from apps.publish.published_item import LAST_PUBLISHED_VERSION
from apps.legal_archive.commands import import_into_legal_archive


logger = logging.getLogger(__name__)

ITEM_PUBLISH = 'publish'
ITEM_CORRECT = 'correct'
ITEM_KILL = 'kill'
item_operations.extend([ITEM_PUBLISH, ITEM_CORRECT, ITEM_KILL])


class BasePublishResource(ArchiveResource):
    """
    Base resource class for "publish" endpoint.
    """

    def __init__(self, endpoint_name, app, service, publish_type):
        self.endpoint_name = 'archive_%s' % publish_type
        self.resource_title = endpoint_name

        self.datasource = {'source': ARCHIVE}

        self.url = 'archive/{}'.format(publish_type)
        self.item_url = item_url

        self.resource_methods = []
        self.item_methods = ['PATCH']

        self.privileges = {'PATCH': publish_type}
        super().__init__(endpoint_name, app=app, service=service)


class BasePublishService(BaseService):
    """
    Base service class for "publish" endpoint
    """

    publish_type = 'publish'
    published_state = 'published'

    non_digital = partial(filter, lambda s: s.get('subscriber_type', '') == SUBSCRIBER_TYPES.WIRE)
    digital = partial(filter, lambda s: (s.get('subscriber_type', '') in {SUBSCRIBER_TYPES.DIGITAL,
                                                                          SUBSCRIBER_TYPES.ALL}))
    takes_package_service = TakesPackageService()
    package_service = PackageService()

    def raise_if_not_marked_for_publication(self, original):
        if original.get('flags', {}).get('marked_for_not_publication', False):
            raise SuperdeskApiError.badRequestError('Cannot publish an item which is marked as Not for Publication')

    def raise_if_invalid_state_transition(self, original):
        if not is_workflow_state_transition_valid(self.publish_type, original[ITEM_STATE]):
            error_message = "Can't {} as item state is {}" if original[ITEM_TYPE] == CONTENT_TYPE.TEXT else \
                "Can't {} as either package state or one of the items state is {}"
            raise InvalidStateTransitionError(error_message.format(self.publish_type, original[ITEM_STATE]))

    def on_update(self, updates, original):
        self.raise_if_not_marked_for_publication(original)
        self.raise_if_invalid_state_transition(original)

        updated = original.copy()
        updated.update(updates)

        takes_package = self.takes_package_service.get_take_package(original)

        if self.publish_type == 'publish':
            # validate if take can be published
            if takes_package and not self.takes_package_service.can_publish_take(
                    takes_package, updates.get(SEQUENCE, original.get(SEQUENCE, 1))):
                raise PublishQueueError.previous_take_not_published_error(
                    Exception("Previous takes are not published."))

            validate_schedule(updated.get('publish_schedule'), takes_package.get(SEQUENCE, 1) if takes_package else 1)

            if original[ITEM_TYPE] != CONTENT_TYPE.COMPOSITE and updates.get(EMBARGO):
                get_resource_service(ARCHIVE).validate_embargo(updated)

        if self.publish_type in ['correct', 'kill']:
            if updates.get(EMBARGO):
                raise SuperdeskApiError.badRequestError("Embargo can't be set after publishing")

            if updates.get('dateline'):
                raise SuperdeskApiError.badRequestError("Dateline can't be modified after publishing")

        validate_item = {'act': self.publish_type, 'type': original['type'], 'validate': updated}
        validation_errors = get_resource_service('validate').post([validate_item])
        if validation_errors[0]:
            raise ValidationError(validation_errors)

        # validate the package if it is one
        package_validation_errors = []
        self._validate_package_contents(original, takes_package, package_validation_errors)
        if len(package_validation_errors) > 0:
            raise ValidationError(package_validation_errors)

        self._set_updates(original, updates, updates.get(config.LAST_UPDATED, utcnow()))
        updates[ITEM_OPERATION] = ITEM_PUBLISH
        convert_task_attributes_to_objectId(updates)

    def on_updated(self, updates, original):
        self.update_published_collection(published_item_id=original[config.ID_FIELD])
        original = get_resource_service(ARCHIVE).find_one(req=None, _id=original[config.ID_FIELD])
        updates.update(original)

        if updates[ITEM_OPERATION] != ITEM_KILL and \
                original.get(ITEM_TYPE) in [CONTENT_TYPE.TEXT, CONTENT_TYPE.PREFORMATTED]:
            get_resource_service('archive_broadcast').on_broadcast_master_updated(updates[ITEM_OPERATION], original)

        get_resource_service('archive_broadcast').reset_broadcast_status(updates, original)
        push_content_notification([updates])
        self._import_into_legal_archive(updates)

    def update(self, id, updates, original):
        """
        Handles workflow of each Publish, Corrected and Killed.
        """
        try:
            user = get_user()
            last_updated = updates.get(config.LAST_UPDATED, utcnow())
            auto_publish = updates.pop('auto_publish', False)

            if original[ITEM_TYPE] == CONTENT_TYPE.COMPOSITE:
                self._publish_package_items(original, updates)

            queued_digital = False
            package = None

            if original[ITEM_TYPE] != CONTENT_TYPE.COMPOSITE:
                # if target_for is set the we don't to digital client.
                if not (updates.get('targeted_for', original.get('targeted_for')) or
                        is_genre(original, BROADCAST_GENRE)):
                    # check if item is in a digital package
                    package = self.takes_package_service.get_take_package(original)

                    if package:
                        queued_digital = self._publish_takes_package(package, updates, original, last_updated)
                    else:
                        '''
                        If type of the item is text or preformatted
                        then item need to be sent to digital subscribers.
                        So, package the item as a take.
                        '''
                        updated = copy(original)
                        updated.update(updates)

                        if original[ITEM_TYPE] in {CONTENT_TYPE.TEXT, CONTENT_TYPE.PREFORMATTED} and \
                                self.sending_to_digital_subscribers(updated):

                            # create a takes package
                            package_id = self.takes_package_service.package_story_as_a_take(updated, {}, None)
                            insert_into_versions(id_=package_id)

                            updates[LINKED_IN_PACKAGES] = updated[LINKED_IN_PACKAGES]
                            package = get_resource_service(ARCHIVE).find_one(req=None, _id=package_id)
                            queued_digital = self._publish_takes_package(package, updates, original, last_updated)

                # queue only text items
                media_type = None
                updated = deepcopy(original)
                updated.update(updates)
                if package:
                    media_type = SUBSCRIBER_TYPES.WIRE

                queued_wire = self.publish(doc=original, updates=updates, target_media_type=media_type)

                queued = queued_digital or queued_wire
                if not queued:
                    logger.error('Nothing is saved to publish queue for story: {} for action: {}'.
                                 format(original[config.ID_FIELD], self.publish_type))

            self._update_archive(original=original, updates=updates, should_insert_into_versions=auto_publish)
            push_notification('item:publish', item=str(id), unique_name=original['unique_name'],
                              desk=str(original.get('task', {}).get('desk', '')),
                              user=str(user.get(config.ID_FIELD, '')))
        except SuperdeskApiError as e:
            raise e
        except KeyError as e:
            raise SuperdeskApiError.badRequestError(
                message="Key is missing on article to be published: {}".format(str(e)))
        except Exception as e:
            logger.exception("Something bad happened while publishing %s".format(id))
            raise SuperdeskApiError.internalError(message="Failed to publish the item: {}".format(str(e)))

    def _publish_takes_package(self, package, updates, original, last_updated):
        """
        Process the takes to form digital master file content and publish.
        :param dict package: Takes package
        :param dict updates: updates for the take
        :param dict original: original takes
        :param datetime.datetime last_updated: datetime for the updates
        :return bool: boolean flag indicating takes package is queued or not
        """

        package_updates = self.process_takes(updates_of_take_to_be_published=updates,
                                             original_of_take_to_be_published=original,
                                             package=package)

        # If the original package is corrected then the next take shouldn't change it back to 'published'
        preserve_state = package.get(ITEM_STATE, '') == CONTENT_STATE.CORRECTED and \
            updates.get(ITEM_OPERATION, ITEM_PUBLISH) == ITEM_PUBLISH

        self._set_updates(package, package_updates, last_updated, preserve_state)
        package_updates.setdefault(ITEM_OPERATION, updates.get(ITEM_OPERATION, ITEM_PUBLISH))
        self._update_archive(package, package_updates)
        '''
        When embargo is lapsed and the article should go to Digital Subscribers the BasePublishService creates a
        Takes Package whose state is draft. In this case, we can't initiate post-publish actions on the Takes Package as
        the package hasn't been published. And post-publish service's get_subscribers() will return empty list.
        Also, logically without publishing a package post-publish actions on the item doesn't make sense.
        That's the reason checking the Takes Package state and invoking the appropriate Publish Service.
        '''
        if package[ITEM_STATE] in [CONTENT_STATE.PUBLISHED, CONTENT_STATE.CORRECTED]:
            package.update(package_updates)
            queued_digital = self.publish(doc=package, updates=None, target_media_type=SUBSCRIBER_TYPES.DIGITAL)
        else:
            package.update(package_updates)
            queued_digital = get_resource_service('archive_publish').publish(doc=package,
                                                                             updates=None,
                                                                             target_media_type=SUBSCRIBER_TYPES.DIGITAL)

        self.update_published_collection(published_item_id=package[config.ID_FIELD])
        self._import_into_legal_archive(package)
        return queued_digital

    def _import_into_legal_archive(self, doc):
        """
        Import into legal archive async
        :param {dict} doc: document to be imported
        """

        if doc.get(ITEM_STATE) != CONTENT_STATE.SCHEDULED:
            kwargs = {
                'doc': doc
            }
            import_into_legal_archive.apply_async(kwargs=kwargs)

    def _publish_package_items(self, package, updates):
        """
        Publishes all items of a package recursively then publishes the package itself
        :param package: package to publish
        :param updates: payload
        """
        items = self.package_service.get_residrefs(package)

        if len(items) == 0 and self.publish_type == ITEM_PUBLISH:
            raise SuperdeskApiError.badRequestError("Empty package cannot be published!")

        removed_items = []
        if self.publish_type in [ITEM_CORRECT, ITEM_KILL]:
            removed_items, added_items = self._get_changed_items(items, updates)
            # we raise error if correction is done on a empty package. Kill is fine.
            if len(removed_items) == len(items) and len(added_items) == 0 and self.publish_type == ITEM_CORRECT:
                raise SuperdeskApiError.badRequestError("Corrected package cannot be empty!")
            items.extend(added_items)

        subscriber_items = {}

        if items:
            archive_publish = get_resource_service('archive_publish')
            for guid in items:
                package_item = super().find_one(req=None, _id=guid)

                if not package_item:
                    raise SuperdeskApiError.badRequestError(
                        "Package item with id: {} does not exist.".format(guid))

                if package_item[ITEM_STATE] not in PUBLISH_STATES:  # if the item is not published then publish it
                    if package_item[ITEM_TYPE] == CONTENT_TYPE.COMPOSITE:
                        # if the item is a package do recursion to publish
                        sub_updates = {i: updates[i] for i in ['state', 'operation'] if i in updates}
                        sub_updates['groups'] = list(package_item['groups'])
                        self._publish_package_items(package_item, sub_updates)
                        self._update_archive(original=package_item, updates=sub_updates,
                                             should_insert_into_versions=False)
                        self.update_published_collection(published_item_id=package_item[config.ID_FIELD])
                    else:
                        # publish the item
                        archive_publish.patch(id=package_item.pop(config.ID_FIELD), updates=package_item)

                    insert_into_versions(id_=guid)

                elif guid in removed_items:
                    # remove the package information from the package item.
                    linked_in_packages = [linked for linked in package_item.get(LINKED_IN_PACKAGES)
                                          if linked.get(PACKAGE) != package.get(config.ID_FIELD)]
                    super().system_update(guid, {LINKED_IN_PACKAGES: linked_in_packages}, package_item)

                package_item = super().find_one(req=None, _id=guid)
                subscribers = self._get_subscribers_for_package_item(package_item)
                self.package_service.update_field_in_package(updates, package_item[config.ID_FIELD],
                                                             config.VERSION, package_item[config.VERSION])

                if package_item[config.ID_FIELD] in removed_items:
                    digital_item_id = None
                else:
                    digital_item_id = self._get_digital_id_for_package_item(package_item)

                self._extend_subscriber_items(subscriber_items, subscribers, package_item, digital_item_id)

            self.publish_package(package, updates, target_subscribers=subscriber_items)

    def _extend_subscriber_items(self, subscriber_items, subscribers, item, digital_item_id):
        """
        Extends the subscriber_items with the given list of subscribers for the item
        :param subscriber_items: The existing list of subscribers
        :param subscribers: New subscribers that item has been published to - to be added
        :param item: item that has been published
        :param digital_item_id: digital_item_id
        """
        item_id = item[config.ID_FIELD]
        for subscriber in subscribers:
            sid = subscriber[config.ID_FIELD]
            item_list = subscriber_items.get(sid, {}).get('items', {})
            item_list[item_id] = digital_item_id
            subscriber_items[sid] = {'subscriber': subscriber, 'items': item_list}

    def _get_changed_items(self, existing_items, updates):
        """
        Returns the added and removed items from existing_items
        :param existing_items: Existing list
        :param updates: Changes
        :return: list of removed items and list of added items
        """
        if 'groups' in updates:
            new_items = self.package_service.get_residrefs(updates)
            removed_items = list(set(existing_items) - set(new_items))
            added_items = list(set(new_items) - set(existing_items))
            return removed_items, added_items
        else:
            return [], []

    def _get_digital_id_for_package_item(self, package_item):
        """
        Finds the digital item id for a given item in a package
        :param package_item: item in a package
        :return string: Digital item id if there's one otherwise id of package_item
        """
        if package_item[ITEM_TYPE] not in [CONTENT_TYPE.TEXT, CONTENT_TYPE.PREFORMATTED]:
            return package_item[config.ID_FIELD]
        else:
            package_item_takes_package_id = self.takes_package_service.get_take_package_id(package_item)
            if not package_item_takes_package_id:
                return package_item[config.ID_FIELD]
            return package_item_takes_package_id

    def _get_subscribers_for_package_item(self, package_item):
        """
        Finds the list of subscribers for a given item in a package
        :param package_item: item in a package
        :return list: List of subscribers
        :return string: Digital item id if there's one otherwise None
        """
        if package_item[ITEM_TYPE] not in [CONTENT_TYPE.TEXT, CONTENT_TYPE.PREFORMATTED]:
            query = {'$and': [{'item_id': package_item[config.ID_FIELD]},
                              {'publishing_action': package_item[ITEM_STATE]}]}
        else:
            package_item_takes_package = self.takes_package_service.get_take_package(package_item)
            if not package_item_takes_package:
                # this item has not been published to digital subscribers so
                # the list of subscribers are empty
                return []

            query = {'$and': [{'item_id': package_item_takes_package[config.ID_FIELD]},
                              {'publishing_action': package_item_takes_package[ITEM_STATE]}]}

        return self._get_subscribers_for_previously_sent_items(query)

    def _set_updates(self, original, updates, last_updated, preserve_state=False):
        """
        Sets config.VERSION, config.LAST_UPDATED, ITEM_STATE in updates document.
        If item is being published and embargo is available then append Editorial Note with 'Embargoed'.

        :param dict original: original document
        :param dict updates: updates related to the original document
        :param datetime last_updated: datetime of the updates.
        :param preserve_state: doesn't update the state: it is used for preserving the state of digital package
        """

        if not preserve_state:
            self.set_state(original, updates)

        updates.setdefault(config.LAST_UPDATED, last_updated)
        self._set_item_expiry(updates, original)

        if original[config.VERSION] == updates.get(config.VERSION, original[config.VERSION]):
            resolve_document_version(document=updates, resource=ARCHIVE, method='PATCH', latest_doc=original)

        if updates.get(EMBARGO, original.get(EMBARGO)) \
                and updates.get('ednote', original.get('ednote', '')).find('Embargo') == -1:
            updates['ednote'] = '{} {}'.format(original.get('ednote', ''), 'Embargoed.').strip()

    def _set_item_expiry(self, updates, original):
        """
        Set the expiry for the item
        :param dict updates: doc on which publishing action is performed
        """
        desk_id = original.get('task', {}).get('desk')
        stage_id = original.get('task', {}).get('stage')
        offset = updates.get(EMBARGO, original.get(EMBARGO)) or \
            updates.get('publish_schedule', original.get('publish_schedule'))

        updates['expiry'] = get_expiry(desk_id, stage_id, offset=offset)

    def _update_archive(self, original, updates, versioned_doc=None, should_insert_into_versions=True):
        """
        Updates the articles into archive collection and inserts the latest into archive_versions.
        Also clears autosaved versions if any.
        :param: versioned_doc: doc which can be inserted into archive_versions
        :param: should_insert_into_versions if True inserts the latest document into versions collection
        """

        self.backend.update(self.datasource, original[config.ID_FIELD], updates, original)

        if should_insert_into_versions:
            if versioned_doc is None:
                insert_into_versions(id_=original[config.ID_FIELD])
            else:
                insert_into_versions(doc=versioned_doc)

        get_component(ItemAutosave).clear(original[config.ID_FIELD])

    def set_state(self, original, updates):
        """
        Set the state of the document based on the action (publish, correction, kill)
        :param dict original: original document
        :param dict updates: updates related to document
        """
        updates['publish_schedule'] = None
        updates[ITEM_STATE] = self.published_state

    def process_takes(self, updates_of_take_to_be_published, package, original_of_take_to_be_published=None):
        """
        Primary rule for publishing a Take in Takes Package is: all previous takes must be published before a take
        can be published.

        Also, generates body_html of the takes package and make sure the metadata for the package is the same as the
        metadata of the take to be published.

        :param dict updates_of_take_to_be_published: updates for the take to be published
        :param dict package: Takes package to publish
        :param dict original_of_take_to_be_published: original of the take to be published
        :return: Takes Package Updates
        """

        takes = self.takes_package_service.get_published_takes(package)
        body_html = updates_of_take_to_be_published.get('body_html',
                                                        original_of_take_to_be_published.get('body_html', ''))
        package_updates = {}

        groups = package.get(GROUPS, [])
        if groups:
            take_refs = [ref for group in groups if group['id'] == 'main' for ref in group.get('refs')]
            sequence_num_of_take_to_be_published = 0

            take_article_id = updates_of_take_to_be_published.get(
                config.ID_FIELD, original_of_take_to_be_published[config.ID_FIELD])

            for r in take_refs:
                if r[GUID_FIELD] == take_article_id:
                    sequence_num_of_take_to_be_published = r[SEQUENCE]
                    r['is_published'] = True
                    break

            if takes and self.published_state != 'killed':
                body_html_list = [take.get('body_html', '') for take in takes]
                if self.published_state == 'published':
                    body_html_list.append(body_html)
                else:
                    body_html_list[sequence_num_of_take_to_be_published - 1] = body_html

                package_updates['body_html'] = '<br>'.join(body_html_list)
            else:
                package_updates['body_html'] = body_html

            metadata_tobe_copied = self.takes_package_service.fields_for_creating_take.copy()
            metadata_tobe_copied.extend(['publish_schedule', 'byline'])
            updated_take = original_of_take_to_be_published.copy()
            updated_take.update(updates_of_take_to_be_published)
            metadata_from = updated_take
            # this rules has changed to use the last published metadata
            # per ticket SD-3885
            # if self.published_state == 'corrected' and len(takes) > 1:
            #     # get the last take metadata only if there are more than one takes
            #     metadata_from = takes[-1]

            for metadata in metadata_tobe_copied:
                if metadata in metadata_from:
                    package_updates[metadata] = metadata_from.get(metadata)

            package_updates[GROUPS] = groups
            self.package_service.update_field_in_package(package_updates,
                                                         original_of_take_to_be_published[config.ID_FIELD],
                                                         config.VERSION,
                                                         updates_of_take_to_be_published[config.VERSION])

        return package_updates

    def publish_package(self, package, updates, target_subscribers):
        """
        Publishes a given non-take package to given subscribers.
        For each subscriber updates the package definition with the wanted_items for that subscriber
        and removes unwanted_items that doesn't supposed to go that subscriber.
        Text stories are replaced by the digital versions.
        :param package: Package to be published
        :param updates: Updates to the package
        :param target_subscribers: List of subscriber and items-per-subscriber
        """
        self._process_publish_updates(package, updates)
        all_items = self.package_service.get_residrefs(package)
        for items in target_subscribers.values():
            updated = deepcopy(package)
            updates_copy = deepcopy(updates)
            updated.update(updates_copy)
            subscriber = items['subscriber']
            wanted_items = [item for item in items['items'] if items['items'].get(item, None)]
            unwanted_items = [item for item in all_items if item not in wanted_items]
            for i in unwanted_items:
                still_items_left = self.package_service.remove_ref_from_inmem_package(updated, i)
                if not still_items_left and self.publish_type != 'correct':
                    # if nothing left in the package to be published and
                    # if not correcting then don't send the package
                    return
            for key in wanted_items:
                self.package_service.replace_ref_in_package(updated, key, items['items'][key])
            self.queue_transmission(updated, [subscriber])

    def _process_publish_updates(self, doc, updates):
        """ Common updates for published items """
        desk = None
        if doc.get('task', {}).get('desk'):
            desk = get_resource_service('desks').find_one(req=None, _id=doc['task']['desk'])
        if not doc.get('ingest_provider'):
            updates['source'] = desk['source'] if desk and desk.get('source', '') \
                else config.DEFAULT_SOURCE_VALUE_FOR_MANUAL_ARTICLES
        updates['pubstatus'] = PUB_STATUS.CANCELED if self.publish_type == 'kill' else PUB_STATUS.USABLE

    def publish(self, doc, updates, target_media_type=None):
        """
        Queue the content for publishing.
        1. Sets the Metadata Properties - source and pubstatus
        2. Get the subscribers.
        3. Update the headline of wire stories with the sequence
        4. Queue the content for subscribers
        5. Queue the content for previously published subscribers if any.
        6. Sends notification if no formatter has found for any of the formats configured in Subscriber.
        7. If not queued and not formatters then raise exception.
        :param dict doc: document to publish
        :param dict updates: updates for the document
        :param str target_media_type: dictate if the doc being queued is a Takes Package or an Individual Article.
                Valid values are - Wire, Digital. If Digital then the doc being queued is a Takes Package and if Wire
                then the doc being queues is an Individual Article.
        :param dict target_subscribers: list of subscribers that document needs to get sent
        :return bool: if content is queued then True else False
        :raises PublishQueueError.item_not_queued_error:
                If the nothing is queued.
        """

        queued = True
        no_formatters = []
        updated = doc.copy()

        # Step 1
        if updates:
            self._process_publish_updates(doc, updates)
            updated.update(updates)

        # Step 2
        subscribers, subscribers_yet_to_receive = self.get_subscribers(doc, target_media_type)

        # Step 3
        if target_media_type == SUBSCRIBER_TYPES.WIRE:
            self._update_headline_sequence(updated)

        # Step 4
        no_formatters, queued = self.queue_transmission(updated, subscribers)

        # Step 5
        if subscribers_yet_to_receive:
            formatters_not_found, queued_new_subscribers = self.queue_transmission(updated, subscribers_yet_to_receive)
            no_formatters.extend(formatters_not_found)
            queued = queued or queued_new_subscribers

        # Step 6
        user = get_user()
        if len(no_formatters) > 0:
            push_notification('item:publish:wrong:format',
                              item=str(doc[config.ID_FIELD]), unique_name=doc['unique_name'],
                              desk=str(doc.get('task', {}).get('desk', '')),
                              user=str(user.get(config.ID_FIELD, '')),
                              formats=no_formatters)

        # Step 7
        if not target_media_type and not queued:
            logger.error('Nothing is saved to publish queue for story: {} for action: {}'.
                         format(doc[config.ID_FIELD], self.publish_type))

        return queued

    def sending_to_digital_subscribers(self, doc):
        """
        Returns False if item has embargo and is in future.
        Returns True if there is a digital subscriber either in the previously sent or in yet to be sent subscribers

        :param doc: document
        :return bool: True if there's at least one
        """

        if doc.get(EMBARGO) and doc.get(EMBARGO) > utcnow():
            return False

        subscribers, subscribers_yet_to_receive = self.get_subscribers(doc, SUBSCRIBER_TYPES.DIGITAL)
        subscribers = list(self.digital(subscribers))
        subscribers_yet_to_receive = list(self.digital(subscribers_yet_to_receive))
        return len(subscribers) > 0 or len(subscribers_yet_to_receive) > 0

    def get_subscribers(self, doc, target_media_type):
        """
        Get subscribers for doc based on target_media_type.
        Override this method in the ArchivePublishService, ArchiveCorrectService and ArchiveKillService
        :param doc: Document to publish/correct/kill
        :param target_media_type: dictate if the doc being queued is a Takes Package or an Individual Article.
                Valid values are - Wire, Digital. If Digital then the doc being queued is a Takes Package and if Wire
                then the doc being queues is an Individual Article.
        :return: (list, list) List of filtered subscriber,
                List of subscribers that have not received item previously (empty list in this case).
        """
        raise NotImplementedError()

    def _get_subscribers_for_previously_sent_items(self, lookup):
        """
        Returns list of subscribers that have previously received the item.
        :param dict lookup: elastic query to filter the publish queue
        :return: list of subscribers
        """
        req = ParsedRequest()
        subscribers = []
        queued_items = get_resource_service('publish_queue').get(req=req, lookup=lookup)
        if queued_items.count():
            subscriber_ids = {queued_item['subscriber_id'] for queued_item in queued_items}
            query = {'$and': [{config.ID_FIELD: {'$in': list(subscriber_ids)}}]}
            subscribers = list(get_resource_service('subscribers').get(req=None, lookup=query))
        return subscribers

    def filter_subscribers(self, doc, subscribers, target_media_type):
        """
        Filter subscribers to whom the current document is going to be delivered.
        :param doc: Document to publish/kill/correct
        :param subscribers: List of Subscribers that might potentially get this document
        :param target_media_type: dictate if the doc being queued is a Takes Package or an Individual Article.
                Valid values are - Wire, Digital. If Digital then the doc being queued is a Takes Package and if Wire
                then the doc being queues is an Individual Article.
        :return: List of of filtered subscriber.
        """
        filtered_subscribers = []
        req = ParsedRequest()
        req.args = {'is_global': True}
        service = get_resource_service('content_filters')
        global_filters = list(service.get(req=req, lookup=None))

        for subscriber in subscribers:
            if target_media_type and subscriber.get('subscriber_type', '') != SUBSCRIBER_TYPES.ALL:
                can_send_takes_packages = subscriber['subscriber_type'] == SUBSCRIBER_TYPES.DIGITAL
                if target_media_type == SUBSCRIBER_TYPES.WIRE and can_send_takes_packages or \
                        target_media_type == SUBSCRIBER_TYPES.DIGITAL and not can_send_takes_packages:
                    continue

            if doc.get('targeted_for'):
                found_match = [t for t in doc['targeted_for'] if t['name'] == subscriber.get('subscriber_type', '')]

                if len(found_match) == 0 and subscriber.get('geo_restrictions'):
                    found_match = [t for t in doc['targeted_for'] if t['name'] == subscriber['geo_restrictions']]
                    if len(found_match) == 0 or found_match[0]['allow'] is False:
                        continue
                elif len(found_match) > 0 and found_match[0]['allow'] is False:
                    continue

            if not self.conforms_global_filter(subscriber, global_filters, doc):
                continue

            if not self.conforms_content_filter(subscriber, doc):
                continue

            filtered_subscribers.append(subscriber)

        return filtered_subscribers

    def queue_transmission(self, doc, subscribers):
        """
        Method formats and then queues the article for transmission to the passed subscribers.
        ::Important Note:: Format Type across Subscribers can repeat. But we can't have formatted item generated once
        based on the format_types configured across for all the subscribers as the formatted item must have a published
        sequence number generated by Subscriber.
        :param dict doc: document to queue for transmission
        :param list subscribers: List of subscriber dict.
        :return : (list, bool) tuple of list of missing formatters and boolean flag. True if queued else False
        """

        try:
            queued = False
            no_formatters = []
            for subscriber in subscribers:
                try:
                    if doc[ITEM_TYPE] not in [CONTENT_TYPE.TEXT, CONTENT_TYPE.PREFORMATTED] and \
                            subscriber.get('subscriber_type', '') == SUBSCRIBER_TYPES.WIRE:
                        # wire subscribers can get only text and preformatted stories
                        continue

                    for destination in subscriber['destinations']:
                        # Step 2(a)
                        formatter = get_formatter(destination['format'], doc)

                        if not formatter:  # if formatter not found then record it
                            no_formatters.append(destination['format'])
                            continue

                        formatted_docs = formatter.format(doc, subscriber)

                        for pub_seq_num, formatted_doc in formatted_docs:
                            publish_queue_item = dict()
                            publish_queue_item['item_id'] = doc['_id']
                            publish_queue_item['item_version'] = doc[config.VERSION]
                            publish_queue_item['formatted_item'] = formatted_doc
                            publish_queue_item['subscriber_id'] = subscriber['_id']
                            publish_queue_item['destination'] = destination
                            publish_queue_item['published_seq_num'] = pub_seq_num
                            publish_queue_item['publish_schedule'] = doc.get('publish_schedule', None)
                            publish_queue_item['unique_name'] = doc.get('unique_name', None)
                            publish_queue_item['content_type'] = doc.get('type', None)
                            publish_queue_item['headline'] = doc.get('headline', None)

                            self.set_state(doc, publish_queue_item)
                            if publish_queue_item.get(ITEM_STATE):
                                publish_queue_item['publishing_action'] = publish_queue_item.get(ITEM_STATE)
                                del publish_queue_item[ITEM_STATE]
                            else:
                                publish_queue_item['publishing_action'] = self.published_state

                            get_resource_service('publish_queue').post([publish_queue_item])
                            queued = True
                except:
                    logger.exception("Failed to queue item for id {} with headline {} for subscriber {}."
                                     .format(doc.get(config.ID_FIELD), doc.get('headline'), subscriber.get('name')))

            return no_formatters, queued
        except:
            raise

    def update_published_collection(self, published_item_id):
        """
        Updates the published collection with the published item.
        Set the last_published_version to false for previous versions of the published items.
        :param: str published_item_id: _id of the document.
        """
        published_item = super().find_one(req=None, _id=published_item_id)
        published_item = copy(published_item)
        get_resource_service('published').update_published_items(published_item_id, LAST_PUBLISHED_VERSION, False)
        get_resource_service('published').post([published_item])

    def conforms_content_filter(self, subscriber, doc):
        """
        Checks if the document matches the subscriber filter
        :param subscriber: Subscriber to get the filter
        :param doc: Document to test the filter against
        :return:
        True if there's no filter
        True if matches and permitting
        False if matches and blocking
        False if doesn't match and permitting
        True if doesn't match and blocking
        """
        content_filter = subscriber.get('content_filter')

        if content_filter is None or 'filter_id' not in content_filter or content_filter['filter_id'] is None:
            return True

        service = get_resource_service('content_filters')
        filter = service.find_one(req=None, _id=content_filter['filter_id'])
        does_match = service.does_match(filter, doc)

        if does_match:
            return content_filter['filter_type'] == 'permitting'
        else:
            return content_filter['filter_type'] == 'blocking'

    def conforms_global_filter(self, subscriber, global_filters, doc):
        """
        Checks if subscriber has a override rule against each of the
        global filter and if not checks if document matches the global filter
        :param subscriber: Subscriber to get if the global filter is overriden
        :param global_filters: List of all global filters
        :param doc: Document to test the global filter against
        :return: True if at least one global filter is not overriden
        and it matches the document
        False if global filter matches the document or all of them overriden
        """
        service = get_resource_service('content_filters')
        gfs = subscriber.get('global_filters', {})
        for global_filter in global_filters:
            if gfs.get(str(global_filter['_id']), True):
                # Global filter applies to this subscriber
                if service.does_match(global_filter, doc):
                    # All global filters behaves like blocking filters
                    return False
        return True

    def _update_headline_sequence(self, doc):
        """ Updates the headline of the text story if there's any sequence value in it """
        if doc.get(SEQUENCE):
            doc['headline'] = '{}={}'.format(doc['headline'], doc.get(SEQUENCE))

    def _validate_package_contents(self, package, takes_package, validation_errors=[]):
        """
        If the item passed is a package this function will ensure that the unpublished content validates and none of
        the content is locked by other than the publishing session, also do not allow any killed or spiked content

        :param package:
        :param takes_package:
        :param validation_errors: validation errors are appended if there are any.
        """
        # Ensure it is the sort of thing we need to validate
        if package[ITEM_TYPE] == CONTENT_TYPE.COMPOSITE and not takes_package and self.publish_type == ITEM_PUBLISH:
            items = self.package_service.get_residrefs(package)

            # make sure package is not scheduled or spiked
            if package[ITEM_STATE] in (CONTENT_STATE.SPIKED, CONTENT_STATE.SCHEDULED):
                validation_errors.append('Package cannot be {}'.format(package[ITEM_STATE]))

            if package.get(EMBARGO):
                validation_errors.append('Package cannot have Embargo')

            if items:
                for guid in items:
                    doc = super().find_one(req=None, _id=guid)

                    if package[ITEM_TYPE] == CONTENT_TYPE.COMPOSITE:
                        digital = self.takes_package_service.get_take_package(doc) or {}
                        self._validate_package_contents(doc, digital, validation_errors)

                    # make sure no items are killed or spiked or scheduled
                    if doc[ITEM_STATE] in (CONTENT_STATE.KILLED, CONTENT_STATE.SPIKED, CONTENT_STATE.SCHEDULED):
                        validation_errors.append('Package cannot contain {} item'.format(doc[ITEM_STATE]))

                    if doc.get(EMBARGO):
                        validation_errors.append('Package cannot have Items with Embargo')

                    # don't validate items that already have published
                    if doc[ITEM_STATE] not in [CONTENT_STATE.PUBLISHED, CONTENT_STATE.CORRECTED]:
                        validate_item = {'act': self.publish_type, 'type': doc[ITEM_TYPE], 'validate': doc}
                        errors = get_resource_service('validate').post([validate_item], headline=True)
                        if errors[0]:
                            validation_errors.extend(errors[0])

                    # check the locks on the items
                    if doc.get('lock_session', None) and package['lock_session'] != doc['lock_session']:
                        validation_errors.extend(['{}: packaged item cannot be locked'.format(doc['headline'])])


superdesk.workflow_state('published')
superdesk.workflow_action(
    name='publish',
    include_states=['fetched', 'routed', 'submitted', 'in_progress', 'scheduled'],
    privileges=['publish']
)

superdesk.workflow_state('scheduled')
superdesk.workflow_action(
    name='schedule',
    include_states=['fetched', 'routed', 'submitted', 'in_progress'],
    privileges=['schedule']
)

superdesk.workflow_action(
    name='deschedule',
    include_states=['scheduled'],
    privileges=['deschedule']
)

superdesk.workflow_state('killed')
superdesk.workflow_action(
    name='kill',
    include_states=['published', 'scheduled', 'corrected'],
    privileges=['kill']
)

superdesk.workflow_state('corrected')
superdesk.workflow_action(
    name='correct',
    include_states=['published', 'corrected'],
    privileges=['correct']
)

superdesk.workflow_action(
    name='rewrite',
    include_states=['published', 'corrected'],
    privileges=['rewrite']
)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource
from superdesk.services import BaseService
from superdesk.metadata.item import metadata_schema
from superdesk.metadata.utils import extra_response_fields, item_url, aggregations
from eve.defaults import resolve_default_values
from eve.methods.common import resolve_document_etag
from superdesk import get_resource_service
from eve.utils import config
from flask import current_app as app

SOURCE = 'ingest'


class IngestResource(Resource):
    schema = {
        'archived': {
            'type': 'datetime'
        }
    }
    schema.update(metadata_schema)
    extra_response_fields = extra_response_fields
    item_url = item_url
    datasource = {
        'search_backend': 'elastic',
        'aggregations': aggregations
    }


class IngestService(BaseService):

    def post_in_mongo(self, docs, **kwargs):
        for doc in docs:
            resolve_default_values(doc, app.config['DOMAIN'][self.datasource]['defaults'])
        self.on_create(docs)
        resolve_document_etag(docs, self.datasource)
        ids = self.backend.create_in_mongo(self.datasource, docs, **kwargs)
        self.on_created(docs)
        return ids

    def put_in_mongo(self, id, document):
        resolve_default_values(document, app.config['DOMAIN'][self.datasource]['defaults'])
        original = self.find_one(req=None, _id=id)
        self.on_replace(document, original)
        resolve_document_etag(document, self.datasource)
        res = self.backend.replace_in_mongo(self.datasource, id, document, original)
        self.on_replaced(document, original)
        return res

    def set_ingest_provider_sequence(self, item, provider):
        """
        Sets the value of ingest_provider_sequence in item.
        :param item: object to which ingest_provider_sequence to be set
        :param provider: ingest_provider object, used to build the key name of sequence
        """
        sequence_number = get_resource_service('sequences').get_next_sequence_number(
            key_name='ingest_providers_{_id}'.format(_id=provider[config.ID_FIELD]),
            max_seq_number=app.config['MAX_VALUE_OF_INGEST_SEQUENCE']
        )
        item['ingest_provider_sequence'] = str(sequence_number)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
from superdesk.metadata.item import Priority


def map_priority(priority):
    """
    Map the IPTC NewsML Priority to Legacy
    :param int priority:
    :return: one character legacy priority
    """
    mapping = {
        Priority.Flash.value: 'f',
        Priority.Urgent.value: 'u',
        Priority.Three_Paragraph.value: 'b'
    }

    return mapping.get(priority, 'r')


def set_subject(category, article):
    """
    Sets the subject code in the odbc_item based on the category, if multiple subject codes are available
    :param category:
    :param article:
    :return:
    """
    subject_reference = None
    # Ensure that there is a subject in the article
    if article.get('subject') and 'qcode' in article['subject'][0]:
        # set the subject reference with the first value, in case we can't do better
        subject_reference = article['subject'][0].get('qcode')
        # we have multiple categories and multiple subjects
        if len(article['subject']) > 1 and len(article.get('anpa_category') or []) > 1:
            # we need to find a more relevant subject reference if possible
            all_categories = superdesk.get_resource_service('vocabularies').find_one(req=None, _id='categories')
            ref_cat = [cat for cat in all_categories['items'] if
                       cat['qcode'].upper() == category['qcode'].upper()]
            # check if there is an associated subject with the category
            if ref_cat and len(ref_cat) == 1 and 'subject' in ref_cat[0]:
                # try to find the lowest level subject that matches
                ref = 0
                for s in article['subject']:
                    if s['qcode'][:2] == ref_cat[0]['subject'][:2]:
                        if int(s['qcode']) > ref:
                            ref = int(s['qcode'])
                if ref > 0:
                    subject_reference = '{0:0>8}'.format(ref)
    return subject_reference



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.activity import add_activity
from eve.utils import ParsedRequest
from flask import g

from superdesk.emails import send_user_mentioned_email

import re
import superdesk


def get_mentions(text):
    """
    Returns the names of desks and users from a given text
    User names starts with '@' and desk names starts with '#'
    """
    user_pattern = re.compile("(^|\s)\@([a-zA-Z0-9-_.]\w+)")
    desk_pattern = re.compile("(^|\s)\#([a-zA-Z0-9-_.]\w+)")
    raw_user_names = set(username for match in re.finditer(user_pattern, text) for username in match.groups())
    raw_desk_names = set(deskname for match in re.finditer(desk_pattern, text) for deskname in match.groups())
    desk_names = [d.replace('_', ' ') for d in raw_desk_names if d.strip()]
    user_names = [u for u in raw_user_names if u.strip()]
    return user_names, desk_names


def send_email_to_mentioned_users(doc, mentioned_users, origin):
    prefs_service = superdesk.get_resource_service('preferences')
    recipients = []
    for user in mentioned_users:
        send_email = prefs_service.email_notification_is_enabled(user_id=user)
        if send_email:
            user_doc = superdesk.get_resource_service('users').find_one(req=None, _id=user)
            recipients.append(user_doc['email'])
    if recipients:
        username = g.user.get('display_name') or g.user.get('username')
        url = '{}/#/workspace?item={}&action=edit&comments={}'.format(origin, doc['item'], doc['_id'])
        send_user_mentioned_email(recipients, username, doc, url)


def get_users(user_names):
    req = ParsedRequest()
    users = superdesk.get_resource_service('users').get(req=req, lookup={'username': {'$in': user_names}})
    users = {user.get('username'): user.get('_id') for user in users}
    return users


def get_desks(desk_names):
    req = ParsedRequest()
    desks = superdesk.get_resource_service('desks').get(req=req, lookup={'name': {'$in': desk_names}})
    desks = {desk.get('name'): desk.get('_id') for desk in desks}
    return desks


def notify_mentioned_users(docs, origin):
    for doc in docs:
        mentioned_users = doc.get('mentioned_users', {}).values()
        if len(mentioned_users) > 0:
            item = superdesk.get_resource_service('archive').find_one(req=None, _id=doc['item'])
            add_activity('user:mention', '', resource=None, type='comment', item=item,
                         comment=doc.get('text'), comment_id=str(doc.get('_id')),
                         notify=mentioned_users)
            send_email_to_mentioned_users(doc, mentioned_users, origin)


def notify_mentioned_desks(docs):
    for doc in docs:
        mentioned_desks = doc.get('mentioned_desks', {}).values()
        if len(mentioned_desks) > 0:
            item = superdesk.get_resource_service('archive').find_one(req=None, _id=doc['item'])
            add_activity('desk:mention', '', resource=None, type='comment', item=item,
                         comment=doc.get('text'), comment_id=str(doc.get('_id')),
                         notify_desks=mentioned_desks)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk import intrinsic_privilege
from .aap_mm_datalayer import AAPMMDatalayer
from apps.io.search_ingest import SearchIngestService, SearchIngestResource


def init_app(app):
    app.data.aapmm = AAPMMDatalayer(app)
    service = SearchIngestService(datasource=None, backend=app.data.aapmm, source='aapmm')
    SearchIngestResource(endpoint_name='aapmm', app=app, service=service)
    intrinsic_privilege(resource_name='aapmm', method=['GET', 'POST'])



"""Content related helpers and utils.
"""

from superdesk.notification import push_notification
from apps.auth import get_user


def push_content_notification(items):
    """Push content:update notification for multiple items.

    It can be also 2 versions of same item in updated handler
    so that we sent event with both old and new desk/stage.

    :param list items: list of items
    """
    ids = {}
    desks = {}
    stages = {}
    for item in items:
        ids[str(item.get('_id', ''))] = 1
        task = item.get('task', {})
        if task.get('desk'):
            desks[str(task.get('desk', ''))] = 1
        if task.get('stage'):
            stages[str(task.get('stage', ''))] = 1
    user = get_user()
    push_notification(
        'content:update',
        user=str(user.get('_id', '')),
        items=ids,
        desks=desks,
        stages=stages
    )



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from apps.search_providers import register_search_provider
from superdesk import intrinsic_privilege

PROVIDER_NAME = 'aapmm'


def init_app(app):
    # Must be imported here as the constant PROVIDER_NAME is referenced by the below modules
    from .aap_mm_datalayer import AAPMMDatalayer
    from .resource import AAPMMResource
    from .service import AAPMMService

    app.data.aapmm = AAPMMDatalayer(app)

    aapmm_service = AAPMMService(datasource=None, backend=app.data.aapmm)
    AAPMMResource(endpoint_name=PROVIDER_NAME, app=app, service=aapmm_service)

    intrinsic_privilege(resource_name=PROVIDER_NAME, method=['GET', 'POST'])


register_search_provider(name=PROVIDER_NAME, fetch_endpoint=PROVIDER_NAME)



#!/usr/bin/env python
# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import json
import signal
import asyncio
import logging
import websockets
import logging.handlers

beat_delay = 5
clients = set()

logger = logging.getLogger(__name__)


@asyncio.coroutine
def client_loop(websocket):
    """Client loop - send it ping every `beat_delay` seconds to keep it alive.

    Nginx would close the connection after 2 minutes of inactivity, that's why.

    Also it does the health check - if socket was closed by client it will
    break the loop and let server deregister the client.

    :param websocket: websocket protocol instance
    """
    pings = 0
    while True:
        yield from asyncio.sleep(beat_delay)
        if not websocket.open:
            break
        pings += 1
        yield from websocket.send(json.dumps({'ping': pings, 'clients': len(websocket.ws_server.websockets)}))


@asyncio.coroutine
def broadcast(message):
    """Broadcast message to all clients.

    :param message: message as it was received - no encoding/decoding.
    """
    logger.debug('broadcast %s' % message)
    for websocket in clients:
        if websocket.open:
            yield from websocket.send(message)


@asyncio.coroutine
def server_loop(websocket):
    """Server loop - wait for message and broadcast it.

    When message is none it means the socket is closed
    and there will be no messages so we break the loop.

    :param websocket: websocket protocol instance
    """
    while True:
        message = yield from websocket.recv()
        if message is None:
            break
        yield from broadcast(message)


def log(message, websocket):
    """Log message with some websocket data like address.

    :param message: message string
    :param websocket: websocket protocol instance
    """
    host, port = websocket.remote_address
    logger.info('%s address=%s:%s' % (message, host, port))


@asyncio.coroutine
def connection_handler(websocket, path):
    """Handle incomming connections.

    When this function returns the session is over and it closes the socket,
    so there must be some loops..

    :param websocket: websocket protocol instance
    :param path: url path used by client - used to identify client/server connections
    """
    if 'server' in path:
        log('server open', websocket)
        yield from server_loop(websocket)
        log('server done', websocket)
    else:
        log('client open', websocket)
        clients.add(websocket)
        yield from client_loop(websocket)
        clients.remove(websocket)
        log('client done', websocket)


def create_server(config):
    """Create websocket server and run it until it gets Ctrl+C or SIGTERM.

    :param config: config dictionary
    """
    try:
        host = config['WS_HOST']
        port = int(config['WS_PORT'])
        loop = asyncio.get_event_loop()
        server = loop.run_until_complete(websockets.serve(connection_handler, host, port))
        loop.add_signal_handler(signal.SIGTERM, loop.stop)
        logger.info('listening on %s:%s' % (host, port))
        loop.run_forever()
    except KeyboardInterrupt:
        pass
    finally:
        logger.info('closing server')
        server.close()
        loop.run_until_complete(server.wait_closed())
        loop.stop()
        loop.run_forever()
        loop.close()


if __name__ == '__main__':
    config = {
        'WS_HOST': '0.0.0.0',
        'WS_PORT': '5100',
    }
    create_server(config)




# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
import urllib3
import urllib
import xml.etree.ElementTree as etree
from superdesk import config
from superdesk.io.iptc import subject_codes
from datetime import datetime
from superdesk.metadata.item import ITEM_TYPE, CONTENT_TYPE, ITEM_STATE, CONTENT_STATE
from superdesk.utc import utc
from superdesk.io.commands.update_ingest import process_iptc_codes
from superdesk.etree import get_text_word_count
from apps.archive.common import generate_unique_id_and_name

# The older content does not contain an anpa category, so we derive it from the
# publication name
pubnames = {
    'International Sport': 'S',
    'Racing': 'R',
    'Parliamentary Press Releases': 'P',
    'Features': 'C',
    'Financial News': 'F',
    'General': 'A',
    'aap Features': 'C',
    'aap International News': 'I',
    'aap Australian Sport': 'S',
    'Australian General News': 'A',
    'Asia Pulse Full': 'I',
    'AFR Summary': 'A',
    'Australian Sport': 'T',
    'PR Releases': 'J',
    'Entertainment News': 'E',
    'Special Events': 'Y',
    'Asia Pulse': 'I',
    'aap International Sport': 'S',
    'Emergency Services': 'A',
    'BRW Summary': 'A',
    'FBM Summary': 'A',
    'aap Australian General News': 'A',
    'International News': 'I',
    'aap Financial News': 'F',
    'Asia Pulse Basic': 'I',
    'Political News': 'P',
    'Advisories': 'V'
}


class AppImportTextArchiveCommand(superdesk.Command):

    option_list = (
        superdesk.Option('--start', '-strt', dest='start_id', required=True),
        superdesk.Option('--user', '-usr', dest='user', required=True),
        superdesk.Option('--password', '-pwd', dest='password', required=True),
        superdesk.Option('--url_root', '-url', dest='url', required=True),
        superdesk.Option('--query', '-qry', dest='query', required=True),
        superdesk.Option('--count', '-c', dest='limit', required=False)
    )

    def run(self, start_id, user, password, url, query, limit):
        print('Starting text archive import at {}'.format(start_id))
        self._user = user
        self._password = password
        self._id = int(start_id)
        self._url_root = url
        self._query = urllib.parse.quote(query)
        if limit is not None:
            self._limit = int(limit)
        else:
            self._limit = None

        self._api_login()

        x = self._get_bunch(self._id)
        while x:
            self._process_bunch(x)
            x = self._get_bunch(self._id)
            if self._limit is not None and self._limit <= 0:
                break

        print('finished text archive import')

    def _api_login(self):
        self._http = urllib3.PoolManager()
        credentials = '?login[username]={}&login[password]={}'.format(self._user, self._password)
        url = self._url_root + credentials
        r = self._http.urlopen('GET', url, headers={'Content-Type': 'application/xml'})
        self._headers = {'cookie': r.getheader('set-cookie')}

        self._anpa_categories = superdesk.get_resource_service('vocabularies').find_one(req=None, _id='categories')

    def _get_bunch(self, id):
        url = self._url_root + \
            'archives/txtarch?search_docs[struct_query]=(DCDATA_ID<{0})&search_docs[query]='.format(id)
        url += self._query
        url += '&search_docs[format]=full&search_docs[pagesize]=500&search_docs[page]=1'
        url += '&search_docs[sortorder]=DCDATA_ID%20DESC'
        print(url)
        retries = 3
        while retries > 0:
            r = self._http.request('GET', url, headers=self._headers)
            if r.status == 200:
                e = etree.fromstring(r.data)
                # print(str(r.data))
                count = int(e.find('doc_count').text)
                if count > 0:
                    print('count : {}'.format(count))
                return e
            else:
                self._api_login()
                retries -= 1
        return None

    def _get_head_value(self, doc, field):
        el = doc.find('dcdossier/document/head/' + field)
        if el is not None:
            return el.text
        return None

    def _addkeywords(self, key, doc, item):
            code = self._get_head_value(doc, key)
            if code:
                if 'keywords' not in item:
                    item['keywords'] = []
                item['keywords'].append(code)

    def _process_bunch(self, x):
        # x.findall('dc_rest_docs/dc_rest_doc')[0].get('href')
        for doc in x.findall('dc_rest_docs/dc_rest_doc'):
            print(doc.get('href'))
            id = doc.find('dcdossier').get('id')
            if int(id) < self._id:
                self._id = int(id)
            item = {}
            item['guid'] = doc.find('dcdossier').get('guid')

            # if the item has been modified in the archive then it is due to a kill
            # there is an argument that this item should not be imported at all
            if doc.find('dcdossier').get('created') != doc.find('dcdossier').get('modified'):
                item[ITEM_STATE] = CONTENT_STATE.KILLED
            else:
                item[ITEM_STATE] = CONTENT_STATE.PUBLISHED

            value = datetime.strptime(self._get_head_value(doc, 'PublicationDate'), '%Y%m%d%H%M%S')
            item['firstcreated'] = utc.normalize(value) if value.tzinfo else value
            item['versioncreated'] = item['firstcreated']

            generate_unique_id_and_name(item)
            item['ingest_id'] = id

            item['source'] = self._get_head_value(doc, 'Agency')

            self._addkeywords('AsiaPulseCodes', doc, item)

            byline = self._get_head_value(doc, 'Byline')
            if byline:
                item['byline'] = byline

            # item['service'] = self._get_head_value(doc,'Service')

            category = self._get_head_value(doc, 'Category')
            if not category:
                publication_name = self._get_head_value(doc, 'PublicationName')
                if publication_name in pubnames:
                    category = pubnames[publication_name]
            if category:
                anpacategory = {}
                anpacategory['qcode'] = category
                for anpa_category in self._anpa_categories['items']:
                    if anpacategory['qcode'].lower() == anpa_category['qcode'].lower():
                        anpacategory = {'qcode': anpacategory['qcode'], 'name': anpa_category['name']}
                        break
                item['anpa_category'] = [anpacategory]

            self._addkeywords('CompanyCodes', doc, item)

            type = self._get_head_value(doc, 'Format')
            if type == 'x':
                item[ITEM_TYPE] = CONTENT_TYPE.TEXT
            elif type == 't':
                item[ITEM_TYPE] = CONTENT_TYPE.PREFORMATTED
            else:
                item[ITEM_TYPE] = CONTENT_TYPE.TEXT

            item['keyword'] = self._get_head_value(doc, 'Keyword')
            item['ingest_provider_sequence'] = self._get_head_value(doc, 'Sequence')

            orginal_source = self._get_head_value(doc, 'Author')
            if orginal_source:
                item['original_source'] = orginal_source

            item['headline'] = self._get_head_value(doc, 'Headline')

            code = self._get_head_value(doc, 'SubjectRefNum')
            if code and len(code) == 7:
                code = '0' + code
            if code and code in subject_codes:
                item['subject'] = []
                item['subject'].append({'qcode': code, 'name': subject_codes[code]})
                try:
                    process_iptc_codes(item, None)
                except:
                    pass

            slug = self._get_head_value(doc, 'SLUG')
            if slug:
                item['slugline'] = slug
            else:
                item['slugline'] = self._get_head_value(doc, 'Keyword')

            # self._addkeywords('Takekey', doc, item)
            take_key = self._get_head_value(doc, 'Takekey')
            if take_key:
                item['anpa_take_key'] = take_key

            self._addkeywords('Topic', doc, item)

            self._addkeywords('Selectors', doc, item)

            el = doc.find('dcdossier/document/body/BodyText')
            if el is not None:
                story = el.text
                if item[ITEM_TYPE] == CONTENT_TYPE.TEXT:
                    story = story.replace('\n   ', '<br><br>')
                    story = story.replace('\n', '<br>')
                    item['body_html'] = story
                else:
                    item['body_html'] = story
                try:
                    item['word_count'] = get_text_word_count(item['body_html'])
                except:
                    pass

            item['pubstatus'] = 'usable'
            # this is required for the archived service additional lookup
            item['item_id'] = item['guid']
            item[config.VERSION] = 1

            res = superdesk.get_resource_service('archived')
            original = res.find_one(req=None, guid=item['guid'])
            if not original:
                item['_id'] = item['guid']
                res.post([item])
            else:
                res.patch(original['_id'], item)

            if self._limit:
                self._limit -= 1
            # print(item)


superdesk.command('app:import_text_archive', AppImportTextArchiveCommand())
